#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# copyBackupData.pl
#
use Net::FTP;
use Socket;
# Used to copy a backup file out or in, i.e. to or from a remove
# server, via FTP
#
# Synopsis:
#   copyBackupData  [--in|--out] --userid=XXXX --password=XXXXX
#      --hostname=XXXXX  --remote-directory=xxxx --remote-filename=XXXXXX

# Format
#    copy <component> <filename> <copy flag>
#         <hostname> <userid>    <password>
#         <remote directory>
#
#    Component name is required
#    Input filename have no extension 
#         Optional when copy to host
#         Required when copy from host
#    Copy flag is require 1 for copy to host 0 otherwise
#    Hostname is require
#    User ID and Password is optional default is anonymous
#    Remote directory is optional
#    All optional input must be a null string if there is no input 
#
# defect    date      reason                                     change flag
# 243       27/10/99  backup directory name change                 @C01
# 315       03/11/99  add -U option on top to make it work         @A01
# 339       03/11/99  audit code                                   @A03
# 377       05/11/99  Make filelist and check host file before     @A02
#                     copy from host
# 504       12/11/99  Create component directory if it doesnt exist@A04
# 600       17/11/99  do not log password                          @D01
# 650       29/11/99  change return code                           @C02
#11057      30/05/01  Check for null array for filename             

use strict;

my (@copyBackupData_opts, $copyBackupData_usage);

use ASM::Common;
use ASM::Exec;
use ASM::Log;

setFacility('backdata');

@copyBackupData_opts = ("in", "out", "userid=s", "password=s",
			"hostname=s", "remote-directory=s",
			"remote-filename=s");
$copyBackupData_usage = " --in|--out --userid=USERID --password=PASSWORD --hostname=HOSTNAME --remote-directory=REMOTE DIRECTORY --remote-filename=REMOTE FILENAME";

commonInit(\@copyBackupData_opts, $copyBackupData_usage);

BEGIN { $^W = 1; }

# Variables set in csm_common.pl or by GetOpt.
use vars qw/
  $backup_filename
  $opt_in
  $opt_out
  $opt_userid
  $opt_password
  $opt_hostname
  $opt_remote_directory
  $opt_remote_filename
/;

  
# verify parameter
if (!$opt_out && !$opt_in) {
  error(MSG_IN_OR_OUT_REQUIRED);
  exit(16);
}
if (!$opt_userid) {
  error(MSG_USERID_REQUIRED);
  exit(21);
}
if (!$opt_password) {
  error(MSG_PASSWORD_REQUIRED);
  exit(22);
}
if (!$opt_hostname) {
  error(MSG_HOSTNAME_REQUIRED);
  exit(57);
}
if (!$opt_remote_directory) {
  error(MSG_REMOTE_DIRECTORY_REQUIRED);
  exit(64);
}
if (!$opt_remote_filename) {
  error(MSG_REMOTE_FILENAME_REQUIRED);
  exit(56);
}

# Verify input Host name is reachable
my $hostnamei = $opt_hostname;
audit(MSG_INPUT_HOSTNAME, $hostnamei);

my @gethbnreturn = gethostbyname( $hostnamei); 
audit(MSG_GETHOSTBYNAME_RESULT, @gethbnreturn);

my $vhostname = $gethbnreturn[0];             # verify by name
audit(MSG_HOST_ADDRESS, $vhostname);
   
if (!$vhostname) { 
   audit(MSG_GET_BY_HOSTNAME_FAILED);
   my @gethbaret = gethostbyaddr($hostnamei, AF_INET); # verify by address
   $vhostname = $gethbaret[0];
   if (!$vhostname) {
      error(MSG_GET_BY_HOSTADDRESS_FAILED, $opt_hostname);
      exit 18;                                     #****** FAIL EXIT ****** 
   }
}    

# Verify filename
if ($opt_out && ! -f $backup_filename) {
   error(MSG_NO_BACKUP_FILE_FOUND);
   exit(62);
}

# 
# Connect to server
#
audit(MSG_CONNECT_TO_SERVER);
my $ftp = Net::FTP->new($vhostname, Passive => 1);
if (!$ftp){
  error(MSG_E65);		# remote host cannot be contacted
  exit(1);
}
audit(MSG_LOGON_TO_SERVER);

if (!$ftp->login($opt_userid, $opt_password)) {
  error(MSG_E66);		# unable to lo into the remote FTP server
  printf STDERR "%s %s\n", $ftp->code(), $ftp->message();
  exit(1);
}

#
# Check and change remote directory   
#
$ftp->cwd($opt_remote_directory) or do
{
   error(MSG_CHDIR_TO_REMOTE_DIR_FAILED, $opt_remote_directory);
   printf STDERR "%s %s\n", $ftp->code(), $ftp->message();
   exit(63);
};

#print getFormattedMessage(MSG_HOST_FILE_LIST) . "\n";
#print (" ",$ftp->ls);
#print "\n" . getFormattedMessage(MSG_END_HOST_FILE_LIST) . "\n";

#
# Copy TO host
#
if ($opt_out)
{
   audit(MSG_COPY_TO_HOST_STARTED, $backup_filename, $vhostname, $opt_remote_directory, $opt_remote_filename, $opt_userid);
   if (!$ftp->put($backup_filename, $opt_remote_filename)) {
     error(MSG_COPY_TO_HOST_FAILED, $backup_filename, $vhostname, $opt_remote_directory, $opt_remote_filename, $opt_userid);
     printf STDERR "%s %s\n", $ftp->code(), $ftp->message();
     exit(1);
   }
   audit(MSG_COPY_TO_HOST_COMPLETED, $backup_filename, $vhostname, $opt_remote_directory, $opt_remote_filename, $opt_userid);
}
else
{
   #
   # Copy FROM host
   #
   audit(MSG_GET_SERVER_FILE, $opt_remote_filename);
   my @filelist = $ftp->ls($opt_remote_filename);       # verify copy Get host file list

  
   if ((@filelist == 0) || ($filelist[0] ne $opt_remote_filename))
   {     # Verify file exists on server
      error(MSG_SERVER_FILE_NOT_FOUND, $opt_remote_filename);
      exit 20; 
   }
   # Retrieve file from remote host
   
   audit(MSG_COPY_FROM_HOST_STARTED, $vhostname, $opt_remote_directory, $opt_remote_filename, $backup_filename, $opt_userid);
   $ftp->get($opt_remote_filename, "${backup_filename}-new") or do
   {
      error(MSG_COPY_FROM_HOST_FAILED, $vhostname, $opt_remote_directory, $opt_remote_filename, $backup_filename, $opt_userid);
      printf STDERR "%s %s\n", $ftp->code(), $ftp->message();
      exit(3);
   };
   unlink($backup_filename);
   renameOrDie("${backup_filename}-new", $backup_filename);
   audit(MSG_COPY_FROM_HOST_COMPLETED, $vhostname, $opt_remote_directory, $opt_remote_filename, $backup_filename, $opt_userid);
}   

$ftp->quit;
exit 0;
