#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# createLinuxUser.pl creates a linux user for CSM belonging to CSMgrp group.
#
# Chi D Nguyen <cdn@us.ibm.com> (5 October 1999)
#
#

#############################################
# First we have our configuration variables #
#############################################

use ASM::Exec;
use ASM::Common;
commonInit(["userid=s", "home_parent=s", "uid=i"],
	   "--userid=CSMUSERID --home_parent=PARENT_DIR_OF_HOME_DIR --uid=USERID");

if (!$opt_userid)
{
    logUsage();
    exit(4);
}
else 
{
    my ($rc,$stdout,$stderr);
    my $exit_rc = 0;
    
    #
    # Check to see if the user is already defined
    #
    $pw = getpwnam $opt_userid;
    if ($pw)
    {
	error(MSG_USER_ALREADY_EXISTS, $opt_userid);
	exit 4;
    }

    #
    # Check to see if uid supplied is in correct format
    #
    if ($opt_uid)
    {
	if ($opt_uid < 0)
	{
	    error(MSG_UID_FORMAT_NOT_VALID, $opt_uid);
	    exit 4;
	}
	$pw = getpwuid $opt_uid;
	if ($pw)
	{
	    error(MSG_UID_DUPLICATE, $pw->name, $opt_uid);
	    exit 4;
	}
    }
    
    #
    # create user's home directories
    #
    my $home_parent = $opt_home_parent || "/home";
    my $home = "$home_parent/$opt_userid";
    my $nfs_user = ($home =~ m@^/net@);               # User defined on nfs mount

    #
    # Create parent directories on local system
    #
    if (!$nfs_user)
    {
	if (! -d $home_parent)
	{
	    mkdir($home_parent, 0755) or do {
		error(MSG_COULD_NOT_CREATE_DIRECTORY, $home_parent);
		trace("Error creating directory $home_parent: $!\n");
		exit 3;
	    };
	}
    }
    else  # NFS user   
    {	
	# Ensure home directory exists
	if (! -d $home) {
		error(MSG_COULD_NOT_OPEN_DIRECTORY, $home);
		trace("Error opening home directory $home: $!\n");
		exit 4;
	    };
    }

    # create user in CSMuser group.
    # make their home directory inside /ftp/home

    $syscmd = "useradd -n -d $home ".(!$nfs_user?"-m  -k /etc/skel/csm":"")." -g $CSMgrp ";
    $syscmd .= "-u $opt_uid " if ($opt_uid);
    $syscmd .= "$opt_userid";
      
    remove_old_lock();		# work around bug
    $rc = runit([$syscmd],\$stdout,\$stderr);
    remove_old_lock();		# work around bug
    
    if ($rc != 0)
    {
	error(MSG_COULD_NOT_CREATE_USER, $opt_userid);
	trace("error creating user $opt_userid: $stdout $stderr");
	exit 4;
    }

    audit(MSG_USER_CREATED, $opt_userid);

    if (!$nfs_user)
    {
	$rc = runit(["chmod 755 $home"],\$stdout,\$stderr);
	if ($rc != 0) 
	{
	    error(MSG_UNABLE_TO_CHMOD, $home, "755");
	    trace("Error chmod'ing 755 $home: $!\n");
	    $exit_rc = 2;
	}
    }
    
    #link_files("/home/ftp/bin", "$home/bin");
    #link_files("/home/ftp/lib", "$home/lib");
    #link_files("/home/ftp/etc", "$home/etc");
    
    # Alias user email to root, since users cannot read their mail
    $rc = runit(["/opt/CSM/scripts/setEmailAlias.pl",
		 "--from=$opt_userid",
		 "--to=root"], \$stdout, \$stderr);
    if ($rc != 0) 
    {
	error(MSG_SENDMAIL_UPDATE_ERROR);
	trace("Errors changing alias: $stdout $stderr\n");
	$exit_rc = 2;
    }

    exit $exit_rc;
}

sub remove_old_lock {
  # Work around bug in useradd.
  # In shadowutils-19990827, if useradd doesn't add a group (-n option,
  # which we are using),  it locks the group file but doesn't unlock it.
  # note: TurboLinux create group.lock 
  # note: Redhat 6.2 creates gshadow.lock and group.lock
  if (-f "/etc/group.lock") {
    chomp(my $pid = `cat /etc/group.lock`);
    if (kill(0,$pid)) {
      # Pid still exists, don't remove
      #print "not removing apparently valid lock\n";
    } else {
      #print "REMOVED OLD LOCK!!!\n";
      unlink("/etc/group.lock");
    }
  }
  if (-f "/etc/gshadow.lock") {
    chomp(my $pid = `cat /etc/gshadow.lock`);
    if (kill(0,$pid)) {
      # Pid still exists, don't remove
      #print "not removing apparently valid lock\n";
    } else {
      #print "REMOVED OLD LOCK!!!\n";
      unlink("/etc/gshadow.lock");
    }
  }
}


#
# Create links to anonymous shell files for guestgroup
#
#
sub link_files
{
    my $odir = shift(@_);
    my $tdir = shift(@_);

    if (! -d $tdir)
    {
	mkdir("$tdir", 0755);
    }

    my @files = ();
    if (!opendir (BINDIR, "$odir/"))
    {
        error(MSG_COULD_NOT_OPEN_DIRECTORY, $odir);
	exit(3);
    }
    @files = grep !/^\.\.?$/, readdir BINDIR;
    closedir BINDIR;

    foreach (@files)
    {
	if (!link("$odir/$_", "$tdir/$_"))
	{
            error(MSG_UNABLE_TO_LINK, "$odir/$_", "$tdir/$_");
	    exit(3);
	}
    }
}
