#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2001. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# emailalert.pl implements email alerts.
#  Dan Poirier <poirier@us.ibm.com> (April 10, 2001)

use Net::Ping;
use lib '/usr/lib/perl5/site_perl';
use LWP::UserAgent;
use HTTP::Status;
#use Sys::Hostname;  # Doesn't seem to pick up changes

use POSIX qw(setsid);
use ASM::Exec;
use ASM::Common;
commonInit( [ "daemon",
	      "once",
	      "reload",
	      "stop",
	      "debug"],
	    "--daemon\n\t --once\n\t --reload\n\t --stop\n");

setFacility('sysuser');
use vars qw! $opt_reload $opt_daemon $opt_once $opt_stop $opt_debug !;

##############################################################################
#
# Check parameters
#
if (! ($opt_reload || $opt_stop || $opt_daemon || $opt_once)) {
  logUsage();
  exit(1);
}

##############################################################################
#
# Define variables
#
my $DEBUG = 0;
$DEBUG = 1 if $opt_debug;

(my $me = $0) =~ s|^.*/||;

my $email_file = "/opt/CSM/data/alert.addresses";
my $data_file = "/opt/CSM/data/alert.data";
my $pid_file = "/opt/CSM/data/emailalert.pid";

# We create this file when the daemon exits normally
my $shutdown_file = "/opt/CSM/data/emailalert.normal.shutdown";

# Only check for unexpected reboot once per execution
my $have_checked_reboot = 0;

# How long to wait between checks -- in seconds
my $interval = 5 * 60;		# 5 minutes
if ($DEBUG) { $interval = 5; }	# 5 seconds if debugging
my $first_interval = 60;	# 1 minute first time (during reboot, usually)

# How frequently a continuing event will trigger noticing it again
my $repeat_interval = 24 * 3600; # 24 hours
if ($DEBUG) { $repeat_interval = 30; } # 30 seconds if debugging

# The next time to wake up and do a check
my $next_time;

# Email addresses to send email to
my @email;

# Event settings
my %events;

# Flags set by signal handlers
my $need_reload = 0;

# The time at which we started the latest iteration
my $now = time;

# A 'user agent' to use in retrieving Web pages
my $ua = new LWP::UserAgent;

# Name we should see for our process in 'ps' output
my $name = 'emailalert';

##############################################################################
#
# Set up signal handling
#

# When we receive a SIGHUP, we should re-read the data files.
# Since it's not safe to do much in a signal handler, we just
# set a flag, and check it before we run the loop again.
$SIG{HUP} = sub { $need_reload = 1; };

# When we receive a SIGTERM, we want to cleanly shutdown.
# This signal handler won't ever return, so just do it.
$SIG{TERM} = \&cleanup_and_exit;

##############################################################################
#
# Handle one-off requests
#
if ($opt_reload) {
  signal_reload();
  exit(0);
}
if ($opt_stop) {
  signal_stop();
  exit(0);
}

##############################################################################
#
# Become a daemon if necessary
#
if ($opt_daemon) {
  # Make sure there's not one running already
  if (alreadyRunning($pid_file, $name)) {
    die getFormattedMessage(MSG_ALERT_ALREADY_RUNNING) . "\n";
  }

  daemonize($pid_file);
}


##############################################################################
#
# The main loop!
#
loadSettings();

# Issue messages in English
ASM::Log::changeLocale("en_US");

if ($opt_once) {
  run_checks();
  cleanup_and_exit();
}

my $first_time = 1;

while (1) {
  $now = time();
  $next_time = ($now + $interval);
  print "Top of loop " . scalar(localtime) . "\n" if $DEBUG;
  if ($need_reload) {
    trace("$me reloading settings");
    loadSettings();
  }
  
  # Skip the first check.
  # This allows a newly booted machine to get up and running.
  if ($first_time) {
    $next_time = $now + $first_interval;
    $first_time = 0;
  } else {
    run_checks();
  }

  print "Going to sleep\n" if $DEBUG;
  # the 'sleep' can be interrupted (by receiving a signal), 
  # so loop until the desired time is actually here.
  while ($next_time > time()) {	
    my $sleep = $next_time - time();
    sleep($sleep) if $sleep > 0;
  }
}

##############################################################################
#
# The various checking routines
#

# See if any file systems are full
# Returns a list:
#  The first value is true if any event is NEW
#  The rest of the values are the messages to send 
sub check_fsfull {
  my ($rc,$stdout,$stderr);

  my $event = 'fsfull';
  if (!isEnabled($event)) {
    return (0);
  }

  my @cmd = ("/bin/df", "--local", "--portability");
  $rc = runit_notrace(\@cmd, \$stdout, \$stderr); # don't trace a command every 5 minutes
  if ($rc) {
    trace("@cmd failed; $stdout $stderr");
    error(MSG_COMMAND_FAILED, join(' ',@cmd), $stdout, $stderr);
    return (0);
  } else {
    my $anynew = 0;
    my @msgs;
    # Okay, see what we've got
    my @lines = split("\n", $stdout);
    shift(@lines);		# discard headings
    foreach my $line (@lines) {
      my @f = split(' ',$line);
      my $percent = $f[4];
      my $mountpoint = $f[5];
      $percent =~ s/%$//;
      if ($percent >= $events{"fsfull.data"}) {
	my $msg = getFormattedMessage(MSG_ALERT_FILE_SYSTEM, $mountpoint, $percent);
	if (&isNew("fsfull.$mountpoint")) {
	  $anynew = 1;
	  $msg .= "*";
	}
	push(@msgs, $msg);
      } else {
	&maybeCleanup("fsfull.$mountpoint");
      }
    }
    return ($anynew, @msgs);
  }
}

sub check_load {
  local(*IN);
  my $event = "load";
  if (!isEnabled($event)) {
    return (0);
  }
  if (open(IN, "</proc/loadavg")) {
    my $line = <IN>;
    close(IN);
    my @f = split(' ',$line);
    my $five_minute = $f[1];
    if ($five_minute >= $events{"$event.data"}) {
      my $msg = getFormattedMessage(MSG_ALERT_CPU_LOAD, $five_minute);
      my $new = isNew($event);
      if ($new) { $msg .= "*"; }
      return ($new, $msg);
    } else {
      maybeCleanup($event);
    }
  }
  return (0);
}

sub check_webserver {
  my $event = "webserver";
  if (!isEnabled($event)) {
    return (0);
  }

  my @urls = qw! http://localhost:80/ http://localhost:8008/ http://localhost:1959/ !;

  my $any_new = 0;
  my @msgs;

  foreach my $url (@urls) {
    print "Fetching $url...\n" if $DEBUG;
    my $request = new HTTP::Request('HEAD', $url);
    my $response = $ua->request($request);
    
    # We basically want to see if we talked to a running Web server.
    # Some of our URL requests will 'fail' because we didn't provide a userid/
    # password, but that's okay.
    my $ok = 1;
    if ($response->code == HTTP::Status::RC_INTERNAL_SERVER_ERROR) {
      $ok = 0;			# could not make network connection
    } elsif ($response->code >= 500 && $response->code <= 599) {
      $ok = 0;			# other server-type problems...
    } elsif ($response->code == HTTP::Status::RC_UNAUTHORIZED) {
      $ok = 1;			# normal, don't bother tracing it
    } elsif (!$response->is_success()) {
      # We probably talked to a server, at least...
      # But let's see just what we got
      trace("Requested `$url', got back " . $response->code . ": " . $response->message);
    }
    if (!$ok) {
      print "Fetching $url...failed\n" if $DEBUG;
      my $msg = getFormattedMessage(MSG_ALERT_WEB_SERVER, $url);
      if (isNew("$event.$url")) {
	$any_new = 1;
	$msg .= "*";
      }
      push(@msgs, $msg);
    } else {
      print "Fetching $url...ok\n" if $DEBUG;
      maybeCleanup("$event.$url");
    }
  }
  print "check_webserver done.\n" if $DEBUG;
  return ($any_new, @msgs);
}
sub check_ping {
  my $event = "ping";
  if (!isEnabled($event)) {
    return (0);
  }
  my $host = $events{"$event.data"};
  if ($host) {
    my $p = Net::Ping->new("icmp");
    if (!$p->ping($host)) {
      my $msg = getFormattedMessage(MSG_ALERT_PING, $host);
      my $new = isNew($event);
      $msg .= "*" if $new;
      return ($new, $msg);
    } else {
      maybeCleanup($event);
    }
  }
		      
  return (0);
}

sub check_memory {
  local(*IN);
  my $event = "memory";
  if (!isEnabled($event)) {
    return (0);
  }
  if (open(IN, "</proc/meminfo")) {
    # Skip first 3 lines
    for (my $i = 0; $i < 3; $i++) { my $x = <IN>; }
    # Remaining lines have format: "Name:   value kB"
    my %mem;
    while (<IN>) {
      if (/^(\w+):\s+(\d+) kB$/) {
	$mem{$1} = $2;
      } else {
	warn "Strange line from /proc/meminfo: $_";
      }
    }
    my $totalmem = $mem{MemTotal} + $mem{SwapTotal};
    my $freemem = $mem{MemFree} + $mem{SwapFree};
    my $usedmem = $totalmem - $freemem;
    my $percent = int (0.5 + 100 * ($usedmem / $totalmem )); # compute and round

    if ($percent >= $events{"$event.data"}) {
      my $msg = getFormattedMessage(MSG_ALERT_MEMORY, $percent);
      trace("the formatted message is: $msg\n");
      my $new = isNew($event);
      $msg .= "*" if $new;
      return ($new,$msg);
    } else {
      maybeCleanup($event);
    }
  }
  return (0);
}

sub check_reboot {
  my $event = "reboot";
  if (!isEnabled($event)) {
    return (0);
  }

  if (!$have_checked_reboot) {
    $have_checked_reboot = 1;
    if (! -f $shutdown_file) {
      return(1,
	     getFormattedMessage(MSG_ALERT_UNEXPECTED_REBOOT) . "*");
    }
    # Clean up - if we don't put this file back when we exit,
    # then we'll know next time that we didn't shut down cleanly
    unlink($shutdown_file);
  }
  return (0);
}

##############################################################################
#
# Utility functions
#

# Check for various events.
# Send email if any problems found
sub run_checks {
  my ($anynew, @msgs);

  my @subs = ( \&check_fsfull, \&check_load,   \&check_webserver,
	       \&check_ping,   \&check_memory, \&check_reboot,
	     );
  foreach my $sub (@subs) {
    my ($new,@m) = &$sub;
    if ($new) { $anynew = 1; }
    push(@msgs, @m);
  }
  if ($anynew) {
    if (@email) {
      trace("$me: sending email to " . join(" ",@email));

      my $subject = getFormattedMessage(MSG_ALERT_SUBJECT, hostname());

      # Figure out charset
      my $charset = getMimeCharset(); # from ASM::Log

      my @msg;			# build message here
      push(@msg, "Mime-Version: 1.0\n");
      push(@msg, "Content-Type: text/plain; charset=$charset\n");
      push(@msg, "Content-Transfer-Encoding: 8bit\n");
      push(@msg, "Subject: $subject\n");
      push(@msg, "To: " . join(" ",@email));
      push(@msg, "\n");
      foreach my $msg (@msgs) {
	push(@msg, "$msg\n");
      }

      # Send message using sendmail since we need to set the headers
      local(*MAIL);
      if (open(MAIL, "| /usr/sbin/sendmail -t")) {
	print(MAIL join('',@msg))
 	    or trace("$me: error writing to mail: $!");
	close(MAIL);
      }
      else {
	trace("$me: failed to run mail: $!\n");
      }
    } else {
      trace("$me: No addresses to send email to.");
    }

    trace("$me: " . join("\n",@msgs));
  }
}

sub loadSettings {
  print "$me: loading settings\n" if $DEBUG;

  # Read in the data files, resetting the variables
  @email = ();
  local(*IN);
  if (open(IN, "<$email_file")) {
    while (<IN>) {
      chomp;
      push(@email, $_);
    }
    close(IN);
  }

  %events = ();
  readProperties($data_file, \%events);
       
  $need_reload = 0;
}

sub cleanup_and_exit {
  print "$me: cleanup_and_exit\n" if $DEBUG;
  if ($opt_daemon) {
    touch($shutdown_file);
    unlink($pid_file);
  }
  exit(0);
}


sub touch {
  my ($filename) = @_;
  local(*OUT);
  openOrDie(OUT, ">$filename");
  close(OUT);
}

sub isEnabled ($) {
  my ($event) = @_;
  return $events{"$event.enabled"};
}

# Event happened.  See if it's been 24 hours since it last
# triggered an email, return true or false.
# Also, if it has been 24 hours, set the last trigger time
# to now.
sub isNew ($) {
  my ($event) = @_;
  # Grab the very latest data
  readProperties($data_file, \%events);
  my $last = $events{"$event.lasttime"};
  my $new = 0;
  if (!$last) {
    $new = 1;
  }
  elsif ( ($now - $last) >= $repeat_interval) {
    $new = 1;
  }
  if ($new) {
    # Create or update the last time in the file
    print "new event $event\n" if $DEBUG;
    $events{"$event.lasttime"} = $now;
    writeProperties($data_file, \%events);
    if (!$opt_daemon) {
      signal_reload();
    }
  }
  return $new;
}

# Event did NOT happen, clean it up if last time has expired
sub maybeCleanup($) {
  my ($event) = @_;
  # Grab the very latest data
  readProperties($data_file, \%events);
  my $last = $events{"$event.lasttime"};
  if ($last && ($now - $last) >= $repeat_interval) {
    delete $events{"$event.lasttime"};
    print "delete event $event\n" if $DEBUG;
    writeProperties($data_file, \%events);
    if (!$opt_daemon) {
      signal_reload();
    }
  }
}

sub send_signal($) {
  my ($signal) = @_;
  # Send a signal to a running daemon to reload its variables or quit
  local(*IN);
  if (open(IN, "<$pid_file")) {
    chomp(my $pid = <IN>);
    close(IN);
    kill $signal, $pid;
  }
}

sub signal_stop {
  send_signal(15);
}
sub signal_reload {
  send_signal(1);
}

sub hostname {
  chomp(my $hostname = `/bin/hostname`);
  return $hostname;
}
