#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# ftpFiles.pl - transfer files using FTP.
#
# Usage: 
#   ftpFiles.pl  --xfer=<get|put> [--filename=<file name>] [--filelist=<file name>] [ --tarcompress=<file name> ] [--svr_name=<server name>] [--localdir=<Local Dir>] [--remotedir=<Remote Dir>] [--user_name=<User Name>] [--pasword=<Password>] [--help]
#
#  Parameters:
#           xfer        = Direction of transfer:  Get or Put remote file(s)
#           filename    = Name of file to transfer
#           filelist    = File which contains a list of files to transfer  (Defaults to all files if filename/fileist options not specified)
#           tarcompress = tar and compress files transferred.
#           svr_name    = Name of remote server for FTP transfer
#           img_dir     = Name of remote server directory
#           user_name   = User Name on remote machine to be used for authentication (Required for NFS/FTP)
#           password    = Password of User Name on remote machine to be used for authentication (Required for NFS/FTP)
#
#               	
# Wayne Gao, <waynegao@us.ibm.com>  (15 October 1999)
#
# defect    date      reason                                     change flag
#

use strict;

my ($eSP_BIN, $rc, $stdout, $stderr, $RC, $ftpFiles_usage, @opts, $MNT_PT, $mounted, $date, $rc);
my ($img_type, $xfer, $filename, $filelist, $tarcompress, $tarafter, $svr_name, $svr_dir, $localdir, $remotedir, $user_name, $password, @cmdopts, $logcmd);
my ($fn, $srcdir, $targetdir, @flist, $fname, $ftp, $newdir, $tarcmd, $out, $cf, @list, $curr, $fpath, $currdir, $ftpstat, @files);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
require "$eSP_BIN/common_install.pl";
use Net::FTP;

setFacility('upgrade');

use vars qw/ $opt_help $opt_img_type $opt_xfer $opt_filename $opt_filelist $opt_tarcompress $opt_tarafter $opt_svr_name $opt_svr_dir /;
use vars qw/ $opt_localdir $opt_remotedir $opt_user_name $opt_password /;

# Parse the arguments
@opts = ("admin=s", "xfer=s", "svr_name=s", "tarcompress=s", "tarafter=s", "localdir=s", "remotedir=s", "user_name=s", "password=s","filename=s", "filelist=s", "help");
$ftpFiles_usage = "\t\tor\nftpFiles.pl --xfer=<get|put|remove> --svr_name=<fq_server_name> [ --remotedir=<path to backup dir on remote server>] [ --localdir=<path to backup dir on local server>] [ --user_name=<name> ] [ --password=<name> ] [ --filename=<file to xfer> | --filelist=<file containing filenames to xfer> ] [ --tarcompress=<name of file to tar/compress> ] [tarafter=<tar files after date> ] [ --admin=<name> ]";
 
commonInit(["xfer=s", "svr_name=s", "tarcompress=s", "tarafter=s", "localdir=s", "remotedir=s", "user_name=s", "password=s","filename=s", "filelist=s"], $ftpFiles_usage);

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
 
   trace("$exitmsg");
   trace("exit $exitrc");
   trace("====== ".`date|tr -d "\n"`." ======");
   if ($exitrc != 0)
   {
      error(MSG_FTP_FILES_FAILED);
   }
   else
   {
      audit(MSG_FTP_FILES_COMPLETED);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals(); 

# Get command line options and check 'em out to see if they're okay.
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   trace("Error: Unknown problem with parsing arguments..");
   exit 2;
}

if ($opt_help)
{
   &printUsage; 
}                                          

if (($opt_xfer ne "get") && ($opt_xfer ne "put") && ($opt_xfer ne "remove"))
{
   trace("Error:  xfer is a required parameter with values: get, put, or remove.");
   &printUsage;
}

if ($opt_svr_name eq "")
{
   trace("Error:  server_name is a required parameter.");
   &printUsage;
}

if ($opt_tarafter ne "" && (!$opt_tarcompress || $opt_xfer ne "put"))
{
    trace("Error: tarafter option requires xfer=put and tarcompress option.");
    &printUsage;
}

$xfer = $opt_xfer;
$filename = $opt_filename;
$filelist = $opt_filelist;
$svr_name = $opt_svr_name;
$tarcompress = $opt_tarcompress;
$localdir = $opt_localdir;
$remotedir = $opt_remotedir;
$user_name = $opt_user_name;
$password = $opt_password;
$tarafter = $opt_tarafter;

$date = `date|tr -d "\n"`;
trace("====== $date ======");

trace("xfer=$xfer");
trace("filename=$filename");
trace("filelist=$filelist");
trace("svr_name=$svr_name");
trace("tarcompress=$tarcompress");
trace("localdir=$localdir");
trace("remotedir=$remotedir");
trace("user_name=$user_name");
trace("tarafter=$tarafter");

# Check if the FTP server is reachable -- Turbo does not process -c option correctly
#$rc = 0xffff & system("ping -c1 $svr_name");
#if ($rc !=0)
#{
#   trace("Error: FTP server $svr_name is un-reachable, ftp failed.");
#   close ;
#   exit 65;
#}
#else
#{
#   trace("+++ FTP server $svr_name is reachable, start ftp session");
#}

$rc = chdir $localdir;
if ($rc != 1)
{
   trace("Output of chdir $localdir command: $stdout $stderr");
   error(MSG_COULD_NOT_CHANGE_DIRECTORY, $localdir);
   &cleanup_exit("chdir to directory $localdir failed.  errno = $!", 69);
}

#
# Retrieve list of files to transfer
#
@flist = ();
if ($filename ne "")
{
   #  Probably would want to normalize the files
   #  so that its relative to local/remote dirs.
   push(@flist, $filename);
}
elsif ($filelist ne "")
{
   if (!open(LST, "< $filelist"))
   {
      error(MSG_COULD_NOT_OPEN_FILE, $filelist);
      &cleanup_exit("Error:  couldn't open file $filelist for reading.", 61);
   }
   while(<LST>)
   {
      chomp($_);
      #  Probably would want to normalize the files
      #  so that its relative to local/remote dirs.
      push(@flist, $_);
   }
   close(LST);
}
else
{
   @flist = ".";
}


#  Some files and directories may be fully qualified.  Check
#  and strip off the leading dirs.
#  Remote system for "get" and for the Local system for "put".

#
# Save list of files to a local file for tar command.
#
if ($tarcompress &&
    $xfer eq "put")
{
   if (!open(TARLIST,"> /tmp/ftptar.list"))
   {
      error(MSG_COULD_NOT_CREATE_FILE, "/tmp/ftptar.list");
      &cleanup_exit("Error:  couldn't create tar file list:  /tmp/ftptar.list", 67);
   }
   while($_ = pop(@flist))
   {
      print TARLIST $_."\n";
   }
   close TARLIST;
}

#
# Access remote FTP server
#
$ftp = Net::FTP->new($svr_name, Passive => 1);
if ($ftp eq undef || !$ftp)
{
   error(MSG_UNABLE_TO_CONNECT, $svr_name);
   &cleanup_exit("Unable to connect to $svr_name: $@", 65);
}
if (!$ftp->login($user_name, $password))
{
   error(MSG_UNABLE_TO_LOGIN, $svr_name, $user_name);
   &cleanup_exit("Unable to log into the remote FTP server using userid: $user_name", 66);
}
$ftp->binary();

#
# Change into remote directory
#
if (!$ftp->cwd($remotedir))
{
   if ($xfer eq "put")                        # Couldn't cd into remote directory, create if put
   {
      $newdir = $ftp->mkdir($remotedir, "RECURSE") or &cleanup_exit("Unable to create remote directory.", 80);
      if (!$ftp->cwd($newdir))
      {
         trace("Remote directory $remotedir NOT created successfully.");
         error(MSG_COULD_NOT_CHANGE_DIRECTORY, $remotedir);
         &cleanup_exit("Unable to access remote directory.", 63);
      }
      else
      {
         trace("Remote directory $remotedir created successfully.");
      }
   }
   else
   {
      error(MSG_COULD_NOT_CHANGE_DIRECTORY, $remotedir);
      &cleanup_exit("Unable to access remote directory.", 63);
   }
}

#
# Saving files as a compressed tar file on remote FTP server.
#
if ($xfer eq "put" && $tarcompress)
{
   $rc = chdir $localdir;
   if ($rc != 1)
   {
      error(MSG_COULD_NOT_CHANGE_DIRECTORY, $localdir);
      trace("Output of chdir $localdir command: $stdout $stderr");
      &cleanup_exit("chdir to directory $localdir failed.  errno = $!", 69);
   }

   trace("Creating archive file $tarcompress.tar.gz on remote system at $remotedir.");
   $tarcmd = "tar -zcP --files-from=/tmp/ftptar.list";
   if ($tarafter)
   {
      $tarcmd .= " --after-date=\'$tarafter\'";
   }
   trace("tar command: $tarcmd");
   $out = \*FD;
   if (!open($out,"$tarcmd |"))
   {
      error(MSG_COULD_NOT_CREATE_FILE, "$tarcompress.tar.gz");
      &cleanup_exit("Could not create tar file: $tarcompress.tar.gz", 67);
   }
   trace("Sending archive to remote system.");
   if (!$ftp->put($out, "$tarcompress.tar.gz"))
   {
      error(MSG_ERROR_SENDING_FILE, "$tarcompress.tar.gz", $svr_name);
      &cleanup_exit("Could not transfer tar file.", 67);
   }
   close $out;
   trace("Completed archive creation on remote system.");
}
elsif ($xfer eq "put")
{
   while ($cf = pop(@flist))
   {
      #  All files should be relative to local/remote dirs
      #  except for the case where all files should be transferred.
      $rc = chdir $localdir;
      if ($rc != 1)
      {
         error(MSG_COULD_NOT_CHANGE_DIRECTORY, $localdir);
         trace("Output of chdir $localdir command: $stdout $stderr");
         &cleanup_exit("chdir to directory $localdir failed.  errno = $!", 69);
      }

      #  See if this is a directory.
      #  For a directory, xfer all files/directories within the directory
      if (chdir($cf))
      {
          trace("$cf is a directory.");
          #  Found a directory
          $ftp->mkdir($cf, 1);
          @list = `ls`;
          foreach $fname (@list)
          {
             chop($fname);
             push(@flist, "$curr/$fname");
          }
      }
      else
      {
         #  Found a regular file.  Transfer it.
         #  May need to add a chdir for the path on the file.
         ( $fpath, $fname ) = ( $cf =~ /^(\S*)\/([^\/]+)$/ );
         if ($fname eq "")
         {
            #  No path on the current filename.
            $fpath = "";
            $fname = $cf;
         }
          
         if ($fpath ne "")
         {
            if ($ftp->cwd($fpath))
            {
               $ftp->mkdir($fpath, 1) or &cleanup_exit("Unable to create directory $fpath on remote server", 67);
               if (!$ftp->cwd($fpath))
               {
                  trace("Remote directory $fpath NOT created successfully.");
                  error(MSG_COULD_NOT_CHANGE_DIRECTORY, $fpath);
                  &cleanup_exit("Unable to access remote directory $fpath on remote server $svr_name", 63);
               }
               else
               {
                  trace("Remote directory $fpath created successfully.");
               }
            }
            chdir $fpath;
         }
            
         trace("Transferring file $fname using FTP's $xfer function.");
         $currdir = $ftp->pwd(); 
         trace("Current remote working dir: $currdir");
         chomp($currdir = `pwd`); 
         trace("Current local working dir: $currdir");
         trace("Putting file: $fname");
         $ftpstat = $ftp->put($fname);
         trace("FTP $xfer returned: $ftpstat.");
          
      }
   }
}
elsif ($xfer eq "get")
{
   while ($cf = pop(@flist))
   {
      trace("Processing file $cf.");
      #  All files should be relative to local/remote dirs
      #  except for the case where all files should be transferred.
      $rc = chdir $localdir;
      if ($rc != 1)
      {
         error(MSG_COULD_NOT_CHANGE_DIRECTORY, $localdir);
         trace("Output of chdir $localdir command: $stdout $stderr");
         &cleanup_exit("chdir to directory $localdir failed.  errno = $!", 69);
      }
   
      if ($tarcompress)
      {
	  $tarcompress = "" if ($tarcompress eq "*");
          trace("Retreiving/Extracting archive file from remote system.");
          $out = \*FD;
          if (!open($out,"| tar --same-owner -zpxf - $tarcompress"))
          {
             error(MSG_COULD_NOT_OPEN_FILE, "tar pipe command");
             &cleanup_exit("Error: could not open tar pipe command: |tar --same-owner -zpxf -.", 67);
          }
          if (!$ftp->get($cf, $out))
          {
             error(MSG_ERROR_RECEIVING_FILE, "$cf", $svr_name);
             &cleanup_exit("Unable to retrieve tar file: $cf from remote server: $svr_name", 67);
          }

          close $out;
          trace("Completed archive extraction from remote system.");
      }
      else
      {
         #  See if this is a directory.
         #  For a directory, xfer all files/directories within the directory
         if ($ftp->cwd($cf))
         {
            trace("$cf is directory.  Retrieving contents.");
            $rc = runit(["mkdir -p $cf"], \$stdout, \$stderr);
            if ($rc != 0)
            {
               trace("Output of mkdir -p $cf command: $stdout $stderr");
               error(MSG_COULD_NOT_CREATE_DIRECTORY, $cf);
               &cleanup_exit("Unable to create local directory $cf. rc = $rc", 63);
            }
            @list = $ftp->ls;
            foreach $fname (@list)
            {
               push(@flist, "$cf/$fname");
            }
         }
         else
         {
            #  Found a regular file.  Transfer it.
            #  May need to add a chdir for the path on the file.
            ( $fpath, $fname ) = ( $cf =~ /^(\S*)\/([^\/]+)$/ );
            if ($fname eq "")
            {
               #  No path on the current filename.
               $fpath = "";
               $fname = $cf;
            }
            if ($fpath ne "")
            {
               if (!$ftp->cwd($fpath))
               {
                  error(MSG_COULD_NOT_CHANGE_DIRECTORY, $fpath);
                  &cleanup_exit("Unable to access remote directory $fpath on remote server $svr_name", 67);
               }
               if (!chdir($fpath))
               {
                  $rc = runit(["mkdir -p $fpath"], \$stdout, \$stderr);
                  if ($rc != 0)
                  {
                     trace("Output of mkdir -p $fpath command: $stdout $stderr");
                     error(MSG_COULD_NOT_CREATE_DIRECTORY, $fpath);
                     &cleanup_exit("Unable to create local directory $fpath. rc = $rc", 63);
                  }
                  $rc = chdir $fpath;
                  if ($rc != 1)
                  {
                     error(MSG_COULD_NOT_CHANGE_DIRECTORY, $fpath);
                     trace("Output of chdir $fpath command: $stdout $stderr");
                     &cleanup_exit("chdir to directory $fpath failed.  errno = $!", 69);
                  }
               }
            }
            
            trace("Transferring file $fname using FTP's $xfer function.");
            $currdir = $ftp->pwd(); 
            trace("Current remote working dir: $currdir");
            chomp($currdir = `pwd`); 
            trace("Current local working dir: $currdir");
            $ftpstat = $ftp->$xfer($fname);
            if ($ftpstat eq undef || !$ftpstat)
            {
               if ($xfer eq "put")
               {
                  error(MSG_ERROR_SENDING_FILE, $fname, $svr_name);
                  &cleanup_exit("Unable to $xfer file $fname.", 82);
               }
               else
               {
                  error(MSG_ERROR_RECEIVING_FILE, $fname, $svr_name);
                  &cleanup_exit("Unable to $xfer file $fname.", 83);
               }
            }
            trace("FTP $xfer returns: $ftpstat.");
         }
      }
   }
}
elsif ($xfer eq "remove")
{
   while ($cf = pop(@flist))
   {
      chomp($cf);
      if ($cf eq "*")
      {
         @files = $ftp->ls() or &cleanup_exit("Unable to get listing of remote directory", 81);
         foreach $fn (@files)
         {
            if (!$ftp->delete($fn))
            {
               error(MSG_COULD_NOT_DELETE_FILE, $fn);
               &cleanup_exit("Unable to delete file $fn", 74);
            }
         }
      }
      else
      {
         if (!$ftp->delete($cf))
         {
            error(MSG_COULD_NOT_DELETE_FILE, $cf);
            &cleanup_exit("Unable to delete file $cf", 74);
         }
      }
   }
}

$ftp->quit;

&cleanup_exit("FTP successful.", 0);
