#!/usr/bin/perl -w 
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#


use ASM::Exec;
use ASM::Common;
use File::Copy;

commonInit( [ "list",
	      "initialize_disk=s",
	      "admin=s"],
	      "--list | --initialize_disk=device --admin=s");

use vars qw! $opt_list $opt_initialize_disk $opt_admin !;


#
# Internal routines
#

sub getMemory();
sub getDrives();
sub getPartitions();
sub getCPUs();
sub initializeDisk();
sub dieWithErrorMsg(@);

#
# Main code
#

my $rc = 0;
my $pid_file = "/opt/CSM/data/initdisk.pid";

if ($opt_list)
{
    #
    # cpus in system
    #
    foreach $i (getCPUs())
    {
	print "cpu: $i\n";
    }
    
    #  
    # memory in system
    #
    print "memory: ", int(getMemory()/1024/1024+.5), "\n";
    
    #
    # drives in system
    #
    my @partitions = getPartitions();
    my %active_drives;
    foreach $i (@partitions)
    {
	my ($drive, $blocks) = split /:/, $i;
	$drive =~ /^([hs]d[a-z]).+/;
	$active_drives{"/dev/".$1} = 1;   
    }
    
    for $i (getDrives())
    {
	my ($drive, $blocks) = split /:/, $i;
	print "disk: $drive:$blocks:", exists($active_drives{$drive})?"active":"inactive","\n";
    }

    if (alreadyRunning($pid_file, "hardware.pl"))
    {
	error(MSG_DISK_INITIALIZATION_IN_PROGRESS);
	$rc = 2;
    }
}
elsif ($opt_initialize_disk)
{
    if (alreadyRunning($pid_file, "hardware.pl"))
    {
	error(MSG_DISK_INITIALIZATION_IN_PROGRESS);
	$rc = 4;
    }

    if ($opt_initialize_disk =~ m%/dev/[sh]d[a-z]%)
    {
	$rc = initializeDisk();
    }
}

exit $rc;

#
#  Returns the memory installed in the system.
#
sub getMemory()
{
    my $memory = "0 kB";

    open MEMINFO, "< /proc/meminfo" or die;
    while(<MEMINFO>)
    {
	if (/^Mem:/)
	{
	    $memory = (split /\s/, $_)[2];
	}
    }
    close MEMINFO;

    return $memory;
}


#
# Method: getDrives
#
# Returns an array with the drives on the system in the format =>  name:blocks
#
sub getDrives()
{
    my @drives = ();

    open DRIVES, "/sbin/sfdisk -s |" or die;
    while(<DRIVES>)
    {
	chomp;
	next if (!m%^/dev/.+%);                                       # Skip non device lines
	my ($device, $blocks) = split /:\s*/, $_;                     # 
	push @drives, "$device:$blocks";
    }
    close DRIVES;
    
    return @drives;
}


#
# Method: getPartitions
#
# Returns an array with the partitions on the system in the format => partition_name:blocks
#
sub getPartitions()
{
    my @partitions = ();

    open PARTITIONS, "< /proc/partitions" or die;
    while(<PARTITIONS>)
    {
	next if (/^\s+$/);                                            # Skip blank lines
	$_ =~ s/^\s+//;                                               # Remove leading whitespace
	my ($major, $minor, $blocks, $name, $rest) = split /\s+/, $_;    # 
	if ($name =~ /^[sh]d[a-z][0-9]+/)
	{
	    push @partitions, "$name:$blocks";
	}

    }
    close PARTITIONS;
    
    return @partitions;
}


#
# Method: getCPUs
#
# Returns an array with the CPUs defined to the system
#
sub getCPUs()
{
    my @cpus = ();
    my %cpu;
    
    open CPUINFO, "< /proc/cpuinfo" or die;
    while(<CPUINFO>)
    {
	if (!/^\s+$/)                                                 # Not a blank line
	{ 
	    my ($tag, $value) = split /\s+:\s+/, $_;
	    $value =~ s/\n//;
	    $cpu{$tag} = $value;
	}
	else                                                         # A blank line
	{
	    push @cpus, $cpu{"model name"};
	    %cpu = ();
	}
    }
    close CPUINFO;
    
    return @cpus;
}




#
# Initializes $opt_initialize_dis
#
#
sub initializeDisk()
{
    $SIG{'CHLD'} = 'IGNORE';
    
    my $rc = 0;
    my $partition = $opt_initialize_disk."5";
    
    # Save our PID!
    local(*OUT);
    openOrDie(OUT, "> $pid_file");
    print OUT "$$\n";
    close(OUT);
    
    # Partition disk
    open(WRITE, "| /sbin/sfdisk $opt_initialize_disk") or dieWithErrorMsg(MSG_ERROR_PARTITIONING_DISK, $opt_initialize_disk);
    {
	$SIG{'PIPE'} = sub{print "Pipe error.\n"; exit(4);};
	print WRITE ",,X;\n";    # create extended partion
	print WRITE ";\n";       
	print WRITE ";\n";
	print WRITE ";\n";
	print WRITE ";\n";       # create full linux partition in extended partition
    }
    close(WRITE) or dieWithErrorMsg(MSG_ERROR_PARTITIONING_DISK, $opt_initialize_disk);
    trace "Disk $opt_initialize_disk partitioned.\n";
    
    # Format Disk
    trace "Formating disk $partition.\n";
    system(("/sbin/mke2fs",$partition)) == 0  or dieWithErrorMsg(MSG_ERROR_FORMATTING_DISK, $partition);
    trace "Disk $partition formated.\n";
    
    # Add new disk to /etc/fstab
    trace "Creating /home2.\n";
    if (! -d "/home2")
    {
	mkdir("/home2", "0755") or dieWithErrorMsg(MSG_COULD_NOT_CREATE_DIRECTORY, "/home2");
    }
    
    trace "Modifying /etc/fstab via /etc/fstab.new\n";

    local *FSTAB;
    local *NEWFSTAB;
    openOrDie(FSTAB, "/etc/fstab");
    openOrDie(NEWFSTAB, "> /etc/fstab.new");
    print NEWFSTAB "";      # remove warning message
    while (<FSTAB>)
    {
	if (/^\#/ || /^\s*$/)
	{
	    printOrDie(NEWFSTAB,$_);
	    next;
	}
	
	my ($fs_spec, $fs_file, $fs_vfstype, $fs_mntops, $fs_freq, $fs_passno) = split;
	next if ($fs_file eq "/home2");
	printOrDie(NEWFSTAB, $_);
    }
    printOrDie(NEWFSTAB, "$partition\t/home2\text2\tdefaults,usrquota,nosuid\t1\t2\n");
    closeOrDie(NEWFSTAB);
    closeOrDie(FSTAB);
    trace "Modified /etc/fstab.new\n";
    
    # Save backup copy of /etc/fstab to /etc/fstab.bk
    trace "Replaceing /etc/fstab with /etc/fstab.new\n";
    copy("/etc/fstab", "/etc/fstab.bk");
    if (!rename("/etc/fstab.new", "/etc/fstab"))
    {
	trace("Unable to rename /etc/fstab.new to /etc/fstab");
	dieWithErrorMsg(MSG_COULD_NOT_RENAME_FILE, "/etc/fstab.new", "/etc/fstab");
    }
    
    # Activate new filesytem
    system(("/bin/mount", "/home2")) == 0 or dieWithErrorMsg(MSG_UNABLE_TO_MOUNT, "/home2");

    audit(AUDIT_DISK_INITIALIZED, $opt_initialize_disk);
    
    return 0;
}


sub dieWithErrorMsg(@)
{
    my @messages = $@;

    error(@messages);
    exit(4);
}

















