#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# hosts.pl - Sets and gets the hosts file.
#
# Paul Schroeder <paulsch@us.ibm.com> (28 September 1999)
#
#

my $hosts_usage = "hosts.pl --list\n"
 . "hosts.pl --set --ip=IPADDR --hostname=HOSTNAME [ --nickname=NICKNAME --nickname=NICKNAME . . . ]\n"
  ."hosts.pl --delete=IPADDR\n"
#  ."hosts.pl --set --multiple=HOST_RECORD[;HOSTRECORD;. . .] \n"
;

use ASM::Common;
commonInit(["list", "set", "ip=s", "hostname=s", "nicknames|nickname=s@", 
#"multiple=s",
"delete=s",
], $hosts_usage);

use strict;
BEGIN { $^W = 1; }

use vars qw! $opt_list $opt_set $opt_multiple $opt_hostname
 @opt_nicknames $opt_ip $opt_delete $hosts_file $script_path !;

if ($opt_list) {
	my @hosts = &getHostList();
	for(@hosts) {
	  print $_, "\n";
	}#for
      }
elsif ($opt_delete) { # Remove a given IP address
  deleteHost($opt_delete);
  checkRequiredHosts();
} 
elsif ($opt_set) {
  if ($opt_ip && $opt_hostname) {
    &setHost($opt_ip, $opt_hostname, @opt_nicknames);
    checkRequiredHosts();
  } 
  else {
    &logUsage();
    exit(1);
  }#if-elsif-else
} else {
  &logUsage();
  exit(1);
}#if-elsif-else

exit(0);


################################
# Lookout!  Subroutines below! #
################################

#
# Ensure that the 'required' hosts are in here
# These are:
#  localhost 
#  the main hostname and IP address
# Without these, WebSphere and IBM HTTP Server can fail
#
sub checkRequiredHosts {

  # Ensure localhost is here
  ensureHost("127.0.0.1", "localhost");
  
  # What's the local hostname?
  chomp(my $hostname = `$script_path/network.pl --get hostname`);
  # And the IP address?
  my (undef, $ip, @rest) = split('#', `$script_path/device.pl --get eth0 --sep='#'`);
  ensureHost($ip,$hostname);
}

sub ensureHost {
  my ($ip,$hostname) = @_;

  my %hash;
  getHostHash(\%hash);

  if (exists($hash{$ip}) && ($hash{$ip} =~ /$hostname/)) {
    # We're okay
  } else {
    setHost($ip,$hostname);
  }
}


# Returns a scalar array of hosts table entry strings.
# Each string contains some things separated by single spaces:
#   IPADDR HOSTNAME1 HOSTNAME2 HOSTNAME3...
# (at least one hostname)
sub getHostList {
	my @host_list;

	openOrDie(HOSTS, "<$hosts_file");
	while(<HOSTS>) {
	        next if /^\s*\#/;  # Skip comment lines
		s/\s+$//;  # remove whitespace from end of string
		my ($ip,@names) = split(' ',$_); # split on all whitespace
		# "canonicalize" to a single space between each part
		push(@host_list, join(" ", $ip, @names));
	}#while
	close(HOSTS);

	return @host_list;
}#getHostList

# Remove a given IP address
sub deleteHost {
  my ($ip) = @_;
  my %hash;
  getHostHash(\%hash);
  delete $hash{$ip};
  saveHostHash(\%hash);
}


# Append a single host to the hosts file.
sub setHost {
	my $ip = shift(@_);
	my $name = shift(@_);
	my @nicks = @_;

	&checkData($ip, $name, @nicks);

	# Set the given IP address to the new hostname(s),
	# whether already there or not
	my %hash;
	getHostHash(\%hash);
	$hash{$ip} = join(" ",$name,@nicks);
	saveHostHash(\%hash);
	audit(MSG_ADDED_HOST_DEFINITION, $name, $ip, join(',',@nicks));
	return;
}#setHost

# Set the given hash to a host hash (see makeHostHash)
# Output: the referenced hash is filled in
#   key=ipaddress
#   value="hostname<space>alias1<space>alias2..."
sub getHostHash {
  my ($href) = @_;
  my @hosts = getHostList();
  foreach (@hosts) {
    my ($ip,@rest) = split(' ',$_);
    $href->{$ip} = join(" ",@rest);
  }
}

# Write a hash of host entries to the host file
sub saveHostHash {
  my ($href) = @_;
  local(*OUT);
  openOrDie(OUT, ">$hosts_file");
  foreach (sort keys %$href) {
    print OUT join("\t", $_, $href->{$_}), "\n";
  }
  close(OUT);
}

sub checkData {
	my $ip = shift(@_);
	my $name = shift(@_);
	my @nicks = @_;

	if (!goodIP($ip)) {
	  error(MSG_HOSTS5);
	  exit(5);
	}

	if (!$name) {
	  error(MSG_E006_NO_HOSTNAME);
	  exit(6);
	}
	
	push(@nicks, $name);
	for(@nicks) {
	  if (!goodHostname($_)) {
	    error(MSG_E58, $_);	# host name is invalid
	    exit(7);
	  }
	}#for
}#checkData

