#!/usr/bin/perl -w
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#


# modifyLinuxUser.pl modifies a linux user for CSM belonging to CSMgrp group.
#

#############################################
# First we have our configuration variables #
#############################################

use ASM::Exec;
use ASM::Common;
use User::pwent;


commonInit(["userid=s", "uid=i"],
	   "--userid=CSMUSERID --uid=USERID");

use vars qw ! $opt_userid $opt_uid $opt_passwd !;


if (!$opt_userid)
{
    logUsage();
    exit(4);
}
else 
{
    my ($rc,$stdout,$stderr);
    my $exit_rc = 0;
    
    #
    # Verify that user is already defined
    # 
    my $pw_user = getpwnam $opt_userid;
    if (!$pw_user)
    {
	error(MSG_NO_SUCH_USER, $opt_userid);
	exit 4;
    }

    #
    # Verify that the user is part of the CSM user group
    #
    my ($gname,$gpasswd,$gid,$remainder) = getgrnam($CSMgrp);
    if ($gid != $pw_user->gid)
    {
	error(MSG_NOT_ASM_USER, $opt_userid);
	exit 4;
    }

    #
    # Discard any values not changing from current settings
    #
    if ($opt_uid &&
	$opt_uid == $pw_user->uid)
    {
	$opt_uid = "";
    }

    # Any settings to change???
    if (!$opt_uid &&
	!$opt_passwd)
    {
	exit 0;
    }

    #
    # Check to see if uid supplied is in correct format or if another user id is assigned to it
    #
    if ($opt_uid)
    {
	if ($opt_uid <= 0)
	{
	    error(MSG_UID_FORMAT_NOT_VALID, $opt_uid);
	    exit 4;
	}

	$pw = getpwuid $opt_uid;
	if ($pw &&
	    $pw->name &&
	    $pw->name ne $opt_userid)
	{
	    error(MSG_UID_DUPLICATE, $pw->name, $opt_uid);
	    exit 4;
	}
    }
    
    #
    # Verify password is at least 6 characters long
    #
    if ($opt_passwd &&
	length($opt_passwd) < 6)
    {
	error(MSG_PASSWORD_LENGTH_NOT_VALID);
	exit 4;
    }
    
    # create user in CSMuser group.
    $syscmd = "usermod ";
    $syscmd .= ($opt_uid)? " -u $opt_uid " : "";
    $syscmd .= ($opt_passwd)? " -p ".crypt($opt_passwd, $opt_passwd) : "";
    $syscmd .= " $opt_userid";
      
    remove_old_lock();		# work around bug
    $rc = runit([$syscmd],\$stdout,\$stderr);
    remove_old_lock();		# work around bug
    
    if ($rc != 0)
    {
	error(MSG_COULD_NOT_MODIFY_USER, $opt_userid);
	trace("error creating user $opt_userid: $stdout $stderr");
	exit 4;
    }

    audit(MSG_USER_ACCOUNT_CHANGED, $opt_userid);

    exit 0;
}

sub remove_old_lock {
  # Work around bug in useradd.
  # In shadowutils-19990827, if useradd doesn't add a group (-n option,
  # which we are using),  it locks the group file but doesn't unlock it.
  # note: TurboLinux create group.lock 
  # note: Redhat 6.2 creates gshadow.lock and group.lock
  if (-f "/etc/group.lock") {
    chomp(my $pid = `cat /etc/group.lock`);
    if (kill(0,$pid)) {
      # Pid still exists, don't remove
      #print "not removing apparently valid lock\n";
    } else {
      #print "REMOVED OLD LOCK!!!\n";
      unlink("/etc/group.lock");
    }
  }
  if (-f "/etc/gshadow.lock") {
    chomp(my $pid = `cat /etc/gshadow.lock`);
    if (kill(0,$pid)) {
      # Pid still exists, don't remove
      #print "not removing apparently valid lock\n";
    } else {
      #print "REMOVED OLD LOCK!!!\n";
      unlink("/etc/gshadow.lock");
    }
  }
}


#
# Create links to anonymous shell files for guestgroup
#
#
sub link_files
{
    my $odir = shift(@_);
    my $tdir = shift(@_);

    if (! -d $tdir)
    {
	mkdir("$tdir", 0755);
    }

    my @files = ();
    if (!opendir (BINDIR, "$odir/"))
    {
        error(MSG_COULD_NOT_OPEN_DIRECTORY, $odir);
	exit(3);
    }
    @files = grep !/^\.\.?$/, readdir BINDIR;
    closedir BINDIR;

    foreach (@files)
    {
	if (!link("$odir/$_", "$tdir/$_"))
	{
            error(MSG_UNABLE_TO_LINK, "$odir/$_", "$tdir/$_");
	    exit(3);
	}
    }
}
