#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

# tcpwrappers.pl - manages ftp/telnet access from specific hosts and networks 
#

use strict;

my (@tcpwrappers_opts, $tcpwrappers_usage, $fn, $daemon);

@tcpwrappers_opts = ("list", "ftpd", "telnetd", "add=s", "del=s", "edit=s", "new=s");
$tcpwrappers_usage = "tcpwrappers.pl --ftpd|telnetd [--list] [--add=entry] [--del=entry] [--edit=entry --new=entry]\n"; 

use ASM::Common;
use ASM::Exec;
use ASM::Log;

use vars qw/ $opt_telnetd $opt_ftpd $opt_list $opt_add $opt_del $opt_edit $allow_file $opt_new /;

commonInit(\@tcpwrappers_opts, $tcpwrappers_usage);
setFacility('security');

$fn = $allow_file;

#
# Verfiy required fields
#
if ($opt_telnetd) {$daemon = "in\.telnetd";}
elsif ($opt_ftpd) {$daemon = "in\.ftpd";}
else
{
    logUsage();
    exit (3);
}

if (!$opt_list && !$opt_add && !$opt_del && !$opt_edit)
{
    logUsage();
    exit(3);
} 

#
# 
#
if ($opt_list) {&getList();}
elsif ($opt_add) {&add();}
elsif ($opt_del) {&del($opt_del);}
elsif ($opt_edit) {&edit($opt_edit, $opt_new);}

exit (0);

# Check syntax of a proposed entry
sub isValidEntry ($) {
  my ($entry) = @_;

  if ($entry =~ /^\s*(.*)\s+EXCEPT\s+(.*)\s*$/i) {
    return isValidEntryPart($1) && isValidEntryPart($2);
  } else {
    return isValidEntryPart($entry);
  }
}

# Check syntax of part of a proposed entry -- the part
# before or after the 'except' (or the whole thing if no 'except')
sub isValidEntryPart ($) {
  my ($entry) = @_;
  $entry =~ s/^\s+//;		# remove any leading whitespace
  $entry =~ s/\s+$//;		# remove any trailing whitespace

  my @subparts = split(/[,\s]/, $entry);
  foreach my $subpart (@subparts) {
    if (!isValidEntrySubpart($subpart)) {
      return 0;
    }
  }
  return 1;
}

sub isValidEntrySubpart ($) {
  my ($entry) = @_;

  if ($entry =~ /^(all|local|unknown|known|paranoid)$/i) {
    return 1;			# okay
  }
  if ($entry =~ /^\./) {
    # A  string  that begins with a `. character. A host
    # name is matched if the last components of its  name
    # match the specified pattern.  For example, the pat
    # tern    '.tue.nl    matches    the    host    name
    # 'wzv.win.tue.nl'.
    if (goodHostname($')) {
      # If the rest of it looks like a valid hostname, it's okay
      return 1;
    }
    trace("$entry should be a domain name but isnt' a valid one");
    return 0;
  }
  if ($entry =~ /\.$/) {
    # A string that ends with a `. character. A host address is matched
    # if its first numeric fields match the given string.  For example,
    # the pattern `131.155. matches the address of (almost) every host on
    # the Eindhoven University network (131.155.x.x).
    my @parts = split("\\.", $`);
    foreach my $part (@parts) {
      if ($part !~ /^\d+$/ || $part < 0 || $part > 255) {
	trace("$entry should be an IP address prefix but isn't");
	return 0;
      }
    }
    return 1;
  }
  if ($entry =~ /^[\d\.]+$/) {
    # Looks like a plain old IP address
    if (!goodIP($entry)) {
      trace("$entry looks like an IP address but isn't a valid one");
      return 0;
    }
    return 1;			# valid IP address
  }
  if ($entry =~ /(.*)\/(.*)/ && goodIP($1) && goodIP($2)) {
    # ipaddress/mask
    return 1;
  }
  if (goodHostname($entry)) {
    return 1;			# plain old hostname
  }
  trace("$entry doesn't look like any valid access control entry format");
  return 0;
}

# Return a list of the current entries
sub getcurrent
{
  my @list;
  openOrDie(FILE, "<$fn");
  while(<FILE>)
    {
      if (/^\s*($daemon.*)/)
	{ 
	  /^\s*$daemon\s*:\s*(.*)\s$/;
	  #($discard,$ipdom) = split(/:/, $1); #take off the preceeding ALL
	  push(@list, $1);
	}
    } #while
  
    close(FILE);
  return @list;
}

#
# Function: getList
#
sub getList
{
    openOrDie(FILE, "<$fn");

    while(<FILE>)
    {
	if (/^\s*($daemon.*)/)
	{ 
	    /^\s*$daemon\s*:\s*(.*)\s$/;
	    #($discard,$ipdom) = split(/:/, $1); #take off the preceeding ALL
	    print $1 . "\n"; 
	}
    } #while

    close(FILE);
  }


#
# Function: add
#
sub add
{
  if (!isValidEntry($opt_add)) {
    error(MSG_BAD_ACCESS_STRING, $opt_add);
    exit(3);
  }

    #print "function: add\n";
  my @current = &getcurrent();
  foreach my $entry (@current) {
    if (lc($entry) eq lc($opt_add)) {
      trace("Entry $opt_add for $daemon is already there");
      error(MSG_DUPLICATE_NOT_ALLOWED, $opt_add);
      exit(3);
    }
  }

    openOrDie(FILE, ">>$fn");
    print FILE $daemon.":".$opt_add."\n";
    close(FILE);
    trace("Add allow access to $daemon for $opt_add.");
}

#
# Function: del
#
sub del
{
   my ($j, $k, @file_array, @new_file);
    #print "function: del\n";
    
    my $delete_line = shift(@_);
    #print "deleting: $delete_line\n";

    openOrDie(FILE, "<$fn");
    @file_array = <FILE>;
    close(FILE);
	
    print $#file_array;
    foreach $j (0 .. $#file_array )
    {
	if ($file_array[$j] =~ /^\s*$daemon.*/)
	{ 
	    $file_array[$j] =~ /^\s*$daemon\s*:\s*(.*)\s$/;
	    #print $1."\n";
	    if (lc($1) eq lc($delete_line))
	    {
		next;
	    }
	}
	push(@new_file, $file_array[$j]);
    } #foreach

    openOrDie(FILE, "> $fn");
    foreach $k (0 .. $#new_file ) {
	print FILE $new_file[$k];
    } #foreach
    close (FILE);

    audit(MSG_REMOVED_ACCESS, $daemon, $delete_line);
}

#
# Function: edit
#
sub edit
{
   my ($j, $k, @file_array, @new_file);
    trace("function: edit\n");

    my $delete_line = shift(@_);
    my $add_line = shift(@_);
    trace("deleting: $delete_line\n");
    trace("adding: $add_line\n");

    openOrDie(FILE, "<$fn");
    @file_array = <FILE>;
    close(FILE);
	
    trace $#file_array;
    foreach $j (0 .. $#file_array )
    {
	if ($file_array[$j] =~ /^\s*$daemon.*/)
	{ 
	    $file_array[$j] =~ /^\s*$daemon\s*:\s*(.*)\s$/;
	    trace $1."\n";
	    if (lc($1) eq lc($delete_line))
	    {
		trace "Matched line\n";
		push(@new_file, "$daemon:".$add_line."\n");
		next;
	    }
	}
	trace "Line not matched.\n";
	push(@new_file, $file_array[$j]);
    } #foreach

    openOrDie(FILE, "> $fn");
    foreach $k (0 .. $#new_file ) {
	print FILE $new_file[$k];
    } #foreach
    close (FILE);

    audit(MSG_ADDED_ACCESS, $daemon, $add_line);
}

