#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
#
# timezone.pl - Sets and gets the time zone for the machine.  Also returns a
#	newline separated list of all possible time zones.
#
# Paul Schroeder <paulsch@us.ibm.com> (28 September 1999)
#
#


@timezone_opts = ("get", "set", "list", "zone=s", "utc", "date=s");
$timezone_usage = <<"EOS";
timezone.pl --get [ --list ] [ --sep=SEP ]
timezone.pl --set [ [ --zone=TIMEZONE ] [ -u, --utc ] ]  [ --date=MMDDhhmm[[CC]YY][.ss] ]
EOS

use ASM::Common;
commonInit(\@timezone_opts, $timezone_usage);

use ASM::Exec;
setFacility('syscfg');

$zone_link = "/etc/localtime";
$zone_dir = "/usr/share/zoneinfo";
$zone_clock_file = "/etc/sysconfig/clock";

my ($rc,$stdout,$stderr);

if($opt_get) {
	if($opt_list) {
		@zones = &getZoneList();
		for(@zones) {
			print $_ . "\n";
		}#for
	} else {
		print &getZone;
		print $sep . "UTC" if ($opt_utc && isUTC());
		print "\n";
	}#if-else
} elsif($opt_set) {
	if($opt_zone) {
		&setZone($opt_zone);
		&setUTC($opt_utc);
	}#if
	if($opt_date) {
		&setDate($opt_date);
	}#if
} else {
	logUsage();
}#if-elsif-else

exit(0);


################################
# Lookout!  Subroutines below! #
################################

# Takes --date argument and hands it to the date command.
sub setDate {
	my $date = shift(@_);

 	(print "Can't set date $date:$!" and exit(4)) if system("date $date");
	# Is system time in UTC?
	my $utc_opt = isUTC() ? "--utc" : "--localtime";
	$rc = runit(["hwclock --systohc $utc_opt"], \$stdout, \$stderr);
	if ($rc) {
	  trace("hwclock failed; $!; rc=$rc\n$stdout\n$stderr");
	  error(MSG_ERROR_SETTING_CLOCK);
	  exit(4);
	}

	# Changed time -- restart cron for safety
	runit(["/etc/rc.d/init.d/crond restart"], \$stdout, \$stderr);

	# And regenerate SSL certificate (and restart Web servers)
	runit(["$script_path/createSelfSignedCert.pl"], \$stdout, \$stderr);
	
	audit(MSG_SET_CLOCK, `date`);
}#setDate

# Set a new time zone.  Basically just create a symlink.
sub setZone {
	my $new_zone = shift(@_);
	my @zones = &getZoneList();

	for(@zones) {
		if($new_zone eq $_) {
			unlink($zone_link);
			symlink($zone_dir . "/" . $new_zone, $zone_link);
			audit(MSG_SET_TIMEZONE, $new_zone);
			return;
		}#if
	}#for
	error(MSG_INVALID_TIMEZONE, $new_zone);
	exit(5);

}#setZone

# Current timezone returned as a string.  If not set properly return empty
# string.
sub getZone {
	my $curr_zone = readlink($zone_link);

	if($curr_zone =~ m|^.*$zone_dir/(.*)|) {
		return $1;
	} else {
		return "";
	}#if-else
}#getZone

# Read the zoneinfo directory  and return the zones in alphabetical order.
sub getZoneList {
	my @zone_list = ();

	if (!open(ZONEINFO, "find $zone_dir -type f -or -type l | grep '^$zone_dir/[A-Z]' | sort |")) {
	  trace("Error running find on $zone_dir: $!\n");
	  error(MSG_ERROR_READING_TIMEZONE_DIRECTORY, $zone_dir);
	  exit(6);
	}

	while(<ZONEINFO>) {
		chomp($zone = substr($_, length($zone_dir) + 1));
		push(@zone_list, $zone);
	}#while

	close(ZONEINFO);

	return @zone_list;
}#getZoneList

# Sets whether or not we use UTC (Greenwhich mean time).
sub setUTC {
	my $utc = shift(@_);
	my $utc_val;

	$utc_val = $utc ? "true" : "false";

	openOrDie(CLOCK, ">$zone_clock_file");

	print CLOCK "UTC=$utc_val\n";
	print CLOCK "ARC=false\n";
	close(CLOCK);
	if ($utc)
	{
	  audit(MSG_SET_SYSTEM_CLOCK_TO_UTC);
	}
	else
	{
	  audit(MSG_SET_SYSTEM_CLOCK_TO_LOCAL);
	}
}#setUTC

# Tells us if UTC is true.
sub isUTC {
	my $utc = 0;

	openOrDie(CLOCK, "<$zone_clock_file");

	while(<CLOCK>) {
		if(/^UTC=(true|yes)/i) {
			$utc = 1;
			last;
		}#if
	}#while
	close(CLOCK);

	return $utc;
}#isUTC
