#!/usr/bin/perl -U
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# xferFiles.pl - transfer files via ftp, nfs, or CD.
#
# Usage:
#   xferFiles.pl --img_type=<local|cd|ftp|nfs> --xfer=<get|put|remove> [--filename=<file name>] [--filelist=<file name>] [ --tarcompress=<file name> ] [--svr_name=<server name>] [ --svr_dir=<server directory] [--localdir=<Local Dir>] [--remotedir=<Remote Dir>] [--user_name=<User Name>] [--pasword=<Password>] [--help]
#
#  Parameters:
#           img_type    = Type of file transfer:   Local Filesystem, CD Drive, FTP networking, NFS networking
#           xfer        = Direction of transfer:  Get, Put, or Remove remote file(s).
#           filename    = Name of file to transfer
#           filelist    = File which contains a list of files to transfer  (Defaults to all files if filename/fileist options not specified)
#           tarcompress = tar and compress "put" files.   untar and compress "get" files 
#           svr_name    = Name of remote server for NFS or FTP transfer
#           svr_dir     = Name of remote server directory to be NFS mounted. (Defaults to mount at root on remote server )
#           localdir    = Name of local directory to be used for get/put of remote files (Defaults to current dir)
#           remotedir   = Name of directory on the CD or the Remote Server (Includes /mnt/cdrom for CD or NFS transfers)
#           user_name   = User Name on remote machine to be used for authentication (Required for NFS/FTP)
#           password    = Password of User Name on remote machine to be used for authentication (Required for NFS/FTP)
#
#
use strict;

my ($eSP_BIN, $rc, $stdout, $stderr, $xferFiles_usage, @opts, $MNT_PT, $mounted);
my ($img_type, $xfer, $filename, $filelist, $tarcompress, $tarafter, $svr_name, $svr_dir, $localdir, $remotedir, $user_name, $password, @cmdopts, $logcmd);
my ($fn, $orgCwd, $srcdir, $targetdir, @flist, $fname);

$eSP_BIN = "/opt/CSM/scripts";

use ASM::Common;
require "$eSP_BIN/common_install.pl";
use Cwd;

setFacility('upgrade');

use vars qw/ $opt_help $opt_img_type $opt_xfer $opt_filename $opt_filelist $opt_tarcompress $opt_tarafter $opt_svr_name $opt_svr_dir /;
use vars qw/ $opt_localdir $opt_remotedir $opt_user_name $opt_password /;

# Parse the arguments

# Parse the arguments
@opts = ("admin=s", "img_type=s", "xfer=s", "filename=s", "filelist=s", "tarcompress=s", "tarafter=s", "svr_name=s", "svr_dir=s", "localdir=s", "remotedir=s", "user_name=s", "password=s","help");
$xferFiles_usage = "\t\tor\nxferFiles.pl --xfer=<get|put|remove> --img_type=<local|cd|nfs|ftp> --svr_name=<fq_server_name> [ --svr_dir=<remote NFS mount point> ] [ --remotedir=<path to backup dir on remote server>] [ --localdir=<path to backup dir on local server>] [ --user_name=<name> ] [ --password=<name> ] [ --filename=<file to xfer> | --filelist=<file containing filenames to xfer> ] [ --tarcompress=<name of file to tar/compress> ] [ -- tarafter=<tar files after date> ] [ --admin=<name> ]";

#commonInit(["admin=s", "img_type=s", "xfer=s", "filename=s", "filelist=s", "tarcompress=s", "tarafter=s", "svr_name=s", "svr_dir=s", "localdir=s", "remotedir=s", "user_name=s", "password=s", "help"], $xferFiles_usage);
commonInit(\@opts, $xferFiles_usage);

$mounted = "";

sub cleanup_exit
{
   my ($exitmsg, $exitrc) = @_;
   trace("orgCwd: $orgCwd");
   chdir($orgCwd) unless (!$orgCwd);
   trace("Mount value: $mounted   $MNT_PT"); 
   if ($mounted)
   {
      trace("Attempting umount of $MNT_PT");
      if (&umountit() != 0)
      {
         error(MSG_UNABLE_TO_UNMOUNT, $MNT_PT);
         trace("Couldn't unmount the mount point: $MNT_PT.  Output of umount: $stdout $stderr");
      }
   }
   trace("$exitmsg");
   trace("exit $exitrc");
   trace("====== ".`date|tr -d "\n"`." ======");
   if ($exitrc != 0)
   {
      error(MSG_XFER_FILES_FAILED);
   }
   else
   {
      audit(MSG_XFER_FILES_COMPLETED);
   }
   exit $exitrc;
}

sub sig_handler
{
   my ($sig) = @_;
   audit(MSG_CAUGHT_SIGNAL, $sig);
   &cleanup_exit("Signal $sig caught.   Exiting..", 1);
}

sub setup_signals
{
   $SIG{'TERM'} = 'sig_handler';
   $SIG{'QUIT'} = 'sig_handler';
   $SIG{'INT'} = 'sig_handler';
   $SIG{'HUP'} = 'sig_handler';

   return( 0 );
}

&setup_signals(); 

# Get command line options and check 'em out to see if they're okay.
 
&GetOptions(@opts);

if ($Getopt::Long::error)
{  # There was an error in parsing the arguments
   &cleanup_exit("Error:  Unknown problem with parsing arguments.", 2);
}

if($opt_help)
{
   &printUsage; 
}

if (($opt_img_type ne "local") && ($opt_img_type ne "cd") && ($opt_img_type ne "nfs") && ($opt_img_type ne "ftp"))
{
   error(MSG_XFER_FILES_FAILED);
   trace("Error:  img_type is a required parameter with values: local, cd, nfs, or ftp.");
   &printUsage;
}

if (($opt_xfer ne "get") && ($opt_xfer ne "put") && ($opt_xfer ne "remove"))
{
   error(MSG_XFER_FILES_FAILED);
   trace("Error:  xfer is a required parameter with values: get, put, or remove.");
   &printUsage;
}

if (($opt_tarafter ne "") && (!$opt_tarcompress || ($opt_xfer ne "put")))
{
   trace("Error: tarafter option requires xfer=put and tarcompress option.");
   error(MSG_XFER_FILES_FAILED);
   &printUsage;
}

# Setup Command parameters
$img_type = $opt_img_type;
$xfer = $opt_xfer;
$filename = $opt_filename;
$filelist = $opt_filelist;
$tarafter = $opt_tarafter;
$tarcompress = $opt_tarcompress;
$svr_name = $opt_svr_name;
$svr_dir = $opt_svr_dir;
$localdir = $opt_localdir;
$remotedir = $opt_remotedir;
$user_name = $opt_user_name;
$password = $opt_password;

trace("====== ".`date|tr -d "\n"`." ======");

trace("img_type=$img_type");
trace("xfer=$xfer");
trace("filename=$filename");
trace("filelist=$filelist");
trace("tarcompress=$tarcompress");
trace("svr_name=$svr_name");
trace("localdir=$localdir");
trace("remotedir=$remotedir");
trace("user_name=$user_name");
trace("tarafter=$tarafter");

# For image type other than ftp, modify /etc/fstab.

$orgCwd = cwd();

# Prepare the local directory
trace("Checking local directory $localdir");
if ( ! -d "$localdir" )
{
   if ($xfer eq "get")
   {
      trace("$localdir does not exist, mkdir $localdir");
      $rc = runit(["mkdir -p $localdir"], \$stdout, \$stderr);
      if ($rc != 0)
      {
         &cleanup_exit("ERROR: Cannot make $localdir($rc).  Output of mkdir: $stdout $stderr", 67);
      }
   }
   else
   {
      &cleanup_exit("ERROR: $localdir does not exist.  No files to transfer.  Aborting transfer!", 69);
   }
}

if ($img_type eq "ftp")
{

   @cmdopts = ("--xfer=$xfer","--svr_name=$svr_name", "--user_name=$user_name", "--password=$password");
   if ($filelist ne "")
   {
      push(@cmdopts, "--filelist=$filelist");
   }
   elsif ($filename ne "")
   {
      push(@cmdopts, "--filename=$filename");
   }
   if ($localdir ne "")
   {
      push(@cmdopts, "--localdir=$localdir");
   }
   if ($remotedir ne "")
   {
      push(@cmdopts, "--remotedir=$remotedir");
   }
   if ($tarcompress)
   {
      push(@cmdopts, "--tarcompress=$tarcompress");
   }
   if ($tarafter)
   {
       push(@cmdopts, "--tarafter=$tarafter");
   }
   trace("Transfering files. Wait....");
   trace("Running command: $eSP_BIN/ftpFiles.pl @cmdopts");
   $rc = runit(["$eSP_BIN/ftpFiles.pl", @cmdopts], \$stdout, \$stderr);
   if ($rc != 0)
   {
      trace("Output of ftpFiles.pl: $stdout $stderr");
      &cleanup_exit("Error: Failed to ftp transfer files!\n", 84);
   }
}
else
{
   #####
   #  Need to add tar/compress support
   #####

   if ($img_type eq "local")
   {
      $MNT_PT = "";
      $MNT_PT = "/" if $remotedir eq "";
   }
   else
   {
      $MNT_PT = "/mnt/cdrom";
   }
   #  Must be Local Filesystem, CD, or NFS transfer

   if ($xfer eq "get")
   {
      #  Transferring files from remote system to local system.
      $targetdir = $localdir;
      if ($remotedir eq "")
      {
	  $srcdir = $MNT_PT;
      }
      else
      {
	  $srcdir = $remotedir;
      }
   }
   else
   {
      # ($xfer eq "put")
      #  Transferring files from local system to remote system.
      if ($img_type eq "cd")
      {
	 &cleanup_exit("Error: Only valid xfer is get for CD!\n", 2);
      }
      else
      {
	  if ($remotedir ne "")
	  {
	     $targetdir = "$MNT_PT/$remotedir";
	  }
	  else
	  {
	     $targetdir = "$MNT_PT";
	  }
	  $srcdir = $localdir;
      }
   }

   #
   # Mount filesystem
   #
   if ($img_type eq "cd")
   {
      trace("+++ mount the CD on $MNT_PT +++");
      $rc = &mountit($img_type);
      if ($rc != 0)
      {
         error(MSG_UNABLE_TO_MOUNT, "CD");
         &cleanup_exit("Error:  Couldn't mount CD", $rc);
      }
      $mounted = "true";
      trace("Successfully mounted on $MNT_PT");
  
   }
   elsif ($img_type eq "nfs")
   {
      if ($svr_name eq "")
      {
	 &cleanup_exit("Error: Server name needed for file transfer.\n", 57);
      }
      if ($svr_dir eq "")
      {
	  &cleanup_exit("Error: export directory needed for file transfer.\n", 70);
      }
      #   mount -t iso9660 -o noauto,ro /dev/cdrom $MNT_PT 2>>$
      trace("+++ nfs mount at $MNT_PT +++");
      $rc = &mountit($img_type, $svr_name, $svr_dir);
      if ($rc != 0)
      {
         error(MSG_UNABLE_TO_MOUNT, $svr_name . ":" . $svr_dir);
         &cleanup_exit("Error:  Couldn't mount $svr_name:$svr_dir", $rc);
      }
      $mounted = "true";
      trace("Successfully mounted on $MNT_PT");
   }
  
   #
   # Create target directory
   #
   if (! -e $targetdir)
   {
      mkdir($targetdir, 0755) and &cleanup_exit("Error: unable to mkdir $targetdir", 72);
   }
  
   #
   # Copy files from source directory to target directory
   #
   trace("copy files from $srcdir to $targetdir");
  
   @flist = ();
   if ($filelist ne "")
   {
      open(LST, "< $filelist") or &cleanup_exit("Error:  couldn't open file $filelist for reading.", 67);
      while(<LST>)
      {
	  chomp($_);
	  push(@flist, $_);
      }
      close(LST);
   }
   elsif ($filename ne "")
   {
      push(@flist, $filename);
   }
   else
   {
      push(@flist, "*");
   }
  
   #
   # Transfer files
   #
   if ($xfer eq "put" && $tarcompress)
   {
      open(TARLIST,"> /tmp/tarfiles.list") or &cleanup_exit("Error:  couldn't create tar file.", 67);
      foreach $fn (@flist)
      {
	  print TARLIST "$fn\n";
      }
      close TARLIST;
      
      trace("Creating archive file on remote system.");
      #   Be sure the current working dir is the target dir.
      chdir($targetdir) or &cleanup_exit("Error with cd to $targetdir, rc $?", 67);
      @cmdopts = ("-z", "-c", "-P", "--file=$targetdir/$tarcompress.tar.gz", "--files-from=/tmp/tarfiles.list");
      if ($tarafter)
      {
	  push(@cmdopts, "--after-date=$tarafter");
      }
      $rc = runit(["tar", @cmdopts], \$stdout, \$stderr);
      if ($rc != 1)
      {
         trace("Output of command: $stdout $stderr");
         &cleanup_exit("Error: could not create tar archive $tarcompress.tar.gz.  errno = $!", 76);
      }
      trace("Completed creation of archive $tarcompress.tar.gz on remote system.");
      chdir($orgCwd);
   }
   elsif ($xfer eq "get" && $tarcompress)
   {
      foreach $fname (@flist)
      {
	  trace("Retreiving/Extracting archive file from remote system $svr_name to localdir $localdir.");
	  #   Be sure the current working dir is the target dir.
          chdir( $targetdir) or &cleanup_exit("Error with cd to $targetdir", 67);
	  $rc = runit(["tar --same-owner -zpxf $srcdir/$fname"], \$stdout, \$stderr);
          if ($rc != 1)
          {
             trace("Output of command: $stdout $stderr");
             &cleanup_exit("Error uncompressing file.  errno = $!", 73);
          }
	  trace("Completed archive extraction from remote system for archive $fname.");
      }
   }
   elsif ($xfer eq "remove")
   {
      foreach $fname (@flist)
      {
	  $rc = runit(["rm -rf $targetdir/$fname"], \$stdout, \$stderr);
          if ($rc != 0)
          {
             trace("Output of command: $stdout $stderr");
             &cleanup_exit("Unable to remove file $targetdir/$fname", 74);
          }
      }
   }
   else
   {
      foreach $fname (@flist)
      {
	 $rc = runit(["cp -drf $srcdir/$fname $targetdir/"], \$stdout, \$stderr);
         if ($rc != 0)
         {
            trace("Output of command: $stdout $stderr");
            &cleanup_exit("Error copying $srcdir/$fname to $targetdir", 75);
         }
      }
   }
   trace("Successfully transferred files from $srcdir to $targetdir.");
}

&cleanup_exit("+++ Successfully transferred files.+++", 0);
