#
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

#
# ASM::Common - config variable and common functions used by the CSM scripts.
#

# BEFORE use'ing this file, Perl scripts should set the following
# variables.  In each case, SCRIPTNAME is the base name of the script,
# minus ".pl".  E.g. for /opt/CSM/scripts/setup.pl, use "setup".
#
#   @SCRIPTNAME_opts = ("opt1", "opt2", ...)
#       Options specific to this script.  see Getopt::Long.
#   $SCRIPTNAME_usage = "SCRIPTNAME>pl [ --opt1 | --opt2 } filename"
#       Usage message specific to this script.  csm_common will
#       add on common options like --admin so you don't have to include
#       them here.

use lib "/opt/CSM/scripts";
use ASM::Log;
use ASM::MessageCodes;
use ASM::Exec;
use POSIX;
use File::Copy;

POSIX::setlocale(&POSIX::LC_ALL, "");

#############################################
# First we have our configuration variables #
#############################################

$| = 1;  # Don't buffer. Flush on every write.


# Common config variables
$sep = " ";  # Default field separator on output
$insep = " ";  # Default field separator for STDIN input
$shadow_file = "/etc/shadow"; # the file used to check passwords
@common_opts = ("action=s", "help", "separator=s", "inseparator=s", "admin=s");  # options common to all scripts
$common_usage = "[--admin=name] [--help] ";  # usage common to all scripts
chop($script_name = `basename $0 .pl`);  # get actual name of the running script
# Save the oommand line for later error messages
my $command_line = "$0 " . join(" ",@ARGV);
my $unique_usage = eval("\$${script_name}_usage");
$CSMAdmin_passwd_file = "/opt/CSM/WebServer/conf/admin.passwd";
$auditcmd = "/opt/CSM/bin/createAuditEntry";

################################
# Let's use our needed modules #
################################

use Getopt::Long;  # This modules parses POSIX compliant long option names.
use User::pwent; # Used for checking passwords

####################################################################
# Here we run the functions which we want to do with every script. #
####################################################################


#############################################
# Here's our common subroutine declarations #
#############################################

# See if a daemon is already running
sub alreadyRunning($$) {
  my $pid_file = shift(@_);	# name of file where PID is saved
  my $name = shift(@_);		# name to look for in ps for double-checking
  local(*IN);
  if (open(IN, "$pid_file")) {
    chomp(my $pid = <IN>);
    close(IN);
    if (kill 0, $pid) {		# this fails if there's no such process
      # there's a process running under that PID
      # Is it us?
      my $ps_output = `/bin/ps www $pid`;
      if ($ps_output =~ /$name/) {
	return 1;		# that's us!
      }
    }
  }
  return 0;			# not running, apparently
}

		 
sub daemonize($) {
  my $pid_file = shift(@_);	# name of file where PID is saved
  
  chdir '/'               or die "Can't chdir to /: $!";
  open STDIN, '/dev/null' or die "Can't read /dev/null: $!";
  if (!$DEBUG) {
    open STDOUT, '>/dev/null'
      or die "Can't write to /dev/null: $!";
  }
  defined(my $pid = fork) or die "Can't fork: $!";
  exit if $pid;
  setsid                  or die "Can't start a new session: $!";
  open STDERR, '>&STDOUT' or die "Can't dup stdout: $!";

  # Save our PID!
  local(*OUT);
  openOrDie(OUT, ">$pid_file");
  print OUT "$$\n";
  close(OUT);
}



# Add provided admin userid from the command line arguments
sub mk_auditcmd()
{
   # determine audit command to use
   #
   if ($opt_admin)
   {
      $auditcmd = "$auditcmd -a $opt_admin";
   }
}

# Log usage error, then return
sub logUsage () {
  my $usage = "$script_name.pl $common_usage" . $unique_usage;
  error(MSG_USAGE_ERROR, $script_name, $usage);
}
# Just print command usage to standard error
sub justPrintUsage () {
  #my $usage = shift(@_);
  my $usage = "$script_name.pl $common_usage" . $unique_usage;
  my $msg = getFormattedMessage(MSG_USAGE_ERROR, $script_name, $usage);
  print STDERR $msg, "\n";
}

# Print command usage to standard error. and exit.
sub printUsage () {
  &justPrintUsage();
  if($opt_help) {
    exit(0);
  } else {
    exit(1);
  }
}#printUsage

# Checks if IP address is valid
sub goodIP ($) {
	my $ip = shift(@_);

	return $ip =~ /^([0-9]{1,3}\.){3}[0-9]{1,3}$/
}#goodIP

# For info about valid hostnames, see:
#   RFC 1034: paraphrased
#      each label is 1-63 characters (the pieces between dots)
#      not case sensitive
#      printed representation might end in "."
#      total length host.domainname maximum 255 chars
#      examples include hyphens in names
#      "The labels must follow the rules for ARPANET host names.  They must
#       start with a letter, end with a letter or digit, and have as
#       interior characters only letters, digits, and hyphen.  There
#       are also some restrictions on the length.  Labels must be 63
#       characters or less."


# Checks if a given hostname label (part between dots) is syntactically valid
sub goodHostLabel ($) {
  my ($label) = @_;
  # Check that it starts with a letter,
  # contains only letters, digits, and hyphens,
  # and ends with a letter or digit,
  # and has length <= 63.
  return ($label =~ /^[a-z][-a-z0-9]*$/i)
      && ($label =~ /[a-z0-9]$/i)
      && (length($label) <= 63)
      ;
}

# Checks if a given host name is syntactically valid
sub goodHostname ($) {
	my ($name) = @_;

	if ($name =~ /^\s*$/) { # an empty string or whitespace
	  return 0;		
	}

	my (@labels) = split(/\./, $name);
	foreach my $label (@labels) {
	  if (! goodHostLabel($label)) {
	    trace("hostname $name has bad label $label");
	    return 0;
	  }
	}
	return 1;

	#return $name =~ /^(\w+\.)*\w+$/;
}#goodHostname

# Checks if a given host name is syntactically valid
sub goodDomain ($) {
  return goodHostname($_[0]);
}#goodDomain

# Gets a user's password.  Only requires and old password for the root
# password.
sub getPass ($) {
        my ($user_id) = @_;
        my $pw = getpwnam($user_id);

# Check to see if the old password is correct.
# We are using shadow passwords here so this is a kludge.

    openOrDie(PASSWDFILE,"$shadow_file");

    while ($line = <PASSWDFILE>) {
      ($tmp,$user_pass,$remainder) = split /:/,$line,3; # get password
      if ($tmp == $pw->name) {  # found the password
        last;
      }
    }

    close(PASSWDFILE);

    return ($user_pass);


} #getPass

# Try to open a file, log a message and die if you fail
sub openOrDie (*;$) {
  my ($handle, $filename) = @_;

  if (!open($handle, $filename)) {
    trace("error opening $filename: $!");
    my $base = $filename;	# try to remove funny Perl characters
    $base =~ s/^[+<> ]+//;	#  and get down to the filename itself
    $base =~ s/|//g;

    # See if we were trying to read it or write it
    if ($filename =~ />/) {
      error(MSG_COULD_NOT_CREATE_FILE, $base);
      exit(3);
    } else {
      error(MSG_COULD_NOT_OPEN_FILE, $base);
      exit(3);
    }
  }
  1;
}

# Try to close a file, log a message and die on failure
sub closeOrDie(*) {
  my ($handle) = @_;
  if (!close($handle)) {
    my $msg = "$!";
    trace("error closing file handle $handle: $msg");
    error(MSG_COULD_NOT_CLOSE_FILE, $msg);
    exit(3);
  }
  1;
}

# Try to print to a file handle, log a message and die on failure
sub printOrDie {
  my ($handle, @args) = @_;
  if (!print($handle @args)) {
    my $msg = "$!";
    trace("error printing to file handle $handle: $msg");
    error(MSG_COULD_NOT_PRINT, $msg);
    exit(3);
  }
  1;
}

sub opendirOrDie (*;$) {
  my ($handle, $filename) = @_;

  if (!opendir($handle, $filename)) {
    trace("error opening directory $filename: $!");
    error(MSG_COULD_NOT_OPEN_DIRECTORY, $filename);
    exit(3);
  }
  1;
}

sub chdirOrDie ($) {
  my ($directory) = @_;
  if (!chdir($directory)) {
    trace("chdir $directory failed, $!");
    error(MSG_COULD_NOT_CHANGE_DIRECTORY, $directory);
    exit(3);
  }
  1;
}

sub goodIPOrDie($) {
  my ($ip) = @_;
  if (!goodIP($ip)) {
    trace("bad ip address $ip");
    error(MSG_IP_ADDRESS_NOT_VALID, $ip);
    exit(3);
  }
  1;
}

sub goodHostnameOrIPOrDie($) {
  my ($host) = @_;
  if (!goodHostname($host) && !goodIP($host)) {
    trace("bad hostname $host");
    error(MSG_E58, $host);	# host name 'xxx' is invalid
    exit(3);
  }
  1;
}

sub renameOrDie ($$) {
    my ($from,$to) = @_;
    if (!rename($from,$to)) {
	trace("rename($from,$to) failed: $!");
	error(MSG_COULD_NOT_RENAME_FILE, $from, $to);
	exit(3);
    }
    1;
}

# Copy one filename to another (see File::Copy)
# on failure, issue error and die
sub copyOrDie($$) {
  my ($from, $to) = @_;
  if (!copy($from,$to)) {
    trace("File::Copy::copy($from,$to) failed: $!");
    error(MSG_COULD_NOT_COPY_FILE, $from, $to);
    exit(3);
  }
  1;
}

# Read a 'properties' file into a hash
# Assumes a simple file without quotes, continued lines, etc
# Does nothing if file doesn't exist or cannot be read.
sub readProperties {
  my ($filename, $hashref) = @_;
  local(*P);
  if (open(P, "<$filename")) {
    while (<P>) {
      next if /^\#/;
      chomp;
      my ($key,$value) = split('=',$_,2);
      $hashref->{$key} = $value;
    }
    close(P);
  }
}

# Write a 'properties' file from a hash
# Creates a simple file without quotes, continued lines, etc
sub writeProperties {
  my ($filename, $hashref) = @_;
  local(*P);
  openOrDie(P, ">$filename");
  foreach my $key (keys %$hashref) {
    my $value = $hashref->{$key};
    printf(P "%s=%s\n", $key, $value);
  }
  close(P);
}

# Common settings
$script_path = "/opt/CSM/scripts";
$setup_device = "eth0";
$setup_data_file = "/opt/CSM/data/csm.data";
$inetd_file = "/etc/inetd.conf";
$run_inet = "/etc/rc.d/init.d/inet reload";
$device_dir = "/etc/sysconfig/network-scripts";
$device_file = $device_dir . "/ifcfg-";
$hostname_file = "/etc/HOSTNAME";
$hosts_file = "/etc/hosts";
$CSMgrp = "CSMuser";  # This is the group name of CSM users.

########## allowdeny.pl and tcpwrappers.pl ########
$allow_file = "/etc/hosts.allow";
$deny_file = "/etc/hosts.deny";

########## backups ################
$backup_filename = "/backup/backup.tar";
# All files matching this pattern are considered as lists of files
# to backup, so different ASM packages can have their own.
$backup_config_pattern = "/opt/CSM/conf/backup/*.ctr";
# All files matching this pattern are considered as scripts to be
# run becore a backup.  This allows applications that don't keep
# their config in files to dump them to files just before the
# backup
$prebackup_script_pattern = "/opt/CSM/conf/backup/*.prebackup";
# All files matching this pattern are considered as scripts to be
# run after a restore.  This allows applications that don't keep
# their config in files to restore them from files just after the
# backup
$postrestore_script_pattern = "/opt/CSM/conf/backup/*.postrestore";

##############################################################################
# Get command line options and check 'em out to see if they're okay.
#&GetOptions(@common_opts, eval("\@::${script_name}_opts"));
#setAdmin($opt_admin) if $opt_admin;
#if($opt_help) {
#  &printUsage();
#}#if
#$sep = $opt_separator if($opt_separator);
#$insep = $opt_inseparator if($opt_inseparator);

sub commonInit ($$) {
  my ($opt_ref, $usage) = @_;

  trace("enter script: $command_line");

  $unique_usage = $usage;

  # Need to catch English errors from Getopt::Long
  {
    my $save_sig_warn = $SIG{__WARN__};

    $SIG{__WARN__} = sub { 
       trace("Warning from Getopt::Long:" . join('',@_)); 
    };

    &GetOptions(@common_opts, @$opt_ref);

    $SIG{__WARN__} = $save_sig_warn;
  }

  setAdmin($opt_admin) if $opt_admin;
  if($opt_help) {
    &printUsage();
  }#if
  $sep = $opt_separator if($opt_separator);
  $insep = $opt_inseparator if($opt_inseparator);
}

##############################################################################
&mk_auditcmd;


# That's all folks

1;
