#-*-perl-*-
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

package ASM::Exec;
require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(runit runit_notrace);

use strict;
use POSIX qw(tmpnam);
use Carp;
use ASM::Log;

=head1 NAME

ASM::Exec - Utility for running a subprogram and capturing its output

=head1 SYNOPSIS

  use ASM::Exec;

  my $rc;
  my $stdout, $stderr;
  my @cmd = ("/bin/ls", "-F");

  $rc = runit(\@cmd, \$stdout, \$stderr);

  print "Return code = %d\n", $rc;
  print "Command printed stdout: $stdout\n";
  print "Command printed stderr: $stderr\n";

  $rc = runit(["/bin/cp", "file1", "file2"], \$stdout, \$stderr);
  $rc = runit(["/bin/mv", "file1", "file2"], \$stdout, \$stderr);

  $rc = runit_notrace(["/bin/passwd", "username", "password"], \$stdout, \$stderr);

=head1 DESCRIPTION

The B<ASM::Exec>  module allows ASM Perl scripts to run subcommands
and easily get the return code, standard output, and standard error.

The B<runit> return code is normally the exit() code of the process
(B<not> the value returned by system()).  However, if the process was
killed by a signal (did not call exit()), then the B<runit> return
code is 255.

NOTE: All commands executed using ASM::Exec::runit are traced using
ASM::Log; you don\'t have to trace them separately.  (Output is not
traced, just the command and its return code.)

Use runit_notrace to avoid tracing commands that have passwords
and stuff (trace whatever you want yourself for those).

=cut

sub runit_notrace($$$) {
  my ($cmdref, $stdout_ref, $stderr_ref) = @_;

  local(*SAVE_STDOUT, *SAVE_STDERR);
  open(SAVE_STDOUT, ">&STDOUT") or croak "Exec::runit: $!";
  open(SAVE_STDERR, ">&STDERR") or croak "Exec::runit: $!";

  my $stdout_filename = tmpnam();
  open(STDOUT, "+>$stdout_filename") or croak "Exec::runit: $!";

  my $stderr_filename = tmpnam();
  open(STDERR, "+>$stderr_filename") or croak "Exec::runit: $!";

  select(STDERR); $| = 1;     # make unbuffered
  select(STDOUT); $| = 1;     # make unbuffered

  my $system_rc = system(@$cmdref);

  close(STDOUT);
  local(*IN);
  open(IN, "<$stdout_filename") or croak "Exec::runit: $!";
  $$stdout_ref = join('', <IN>);
  close(IN) or die "Exec::runit: $!";

  close(STDERR);
  open(IN, "<$stderr_filename") or croak "Exec::runit: $!";
  $$stderr_ref = join('', <IN>);
  close(IN) or die "Exec::runit: $!";

  open(STDOUT, ">&SAVE_STDOUT");
  open(STDERR, ">&SAVE_STDERR");

  unlink($stdout_filename);
  unlink($stderr_filename);

  my $rc = $system_rc >> 8;
  if (0 != ($system_rc & 255)) {
    $rc = 255;			# killed by signal
  }
  return $rc;
}

sub runit ($$$) {
  my ($cmdref, $stdout_ref, $stderr_ref) = @_;

  trace("will run COMMAND={" . join(" ",@$cmdref) . "}");
  my $rc = runit_notrace($cmdref, $stdout_ref, $stderr_ref);
  trace("completed COMMAND={" . join(" ",@$cmdref) . "}, return code = $rc");

  return $rc;
}

1;
