#-*-perl-*-
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

package ASM::Log;
require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(setFacility setAdmin setObject trace error audit
	       getFormattedMessage getMimeCharset changeLocale);

# use Sys::Syslog generates Warnings when -w is in effect
# save setting and turn it off temporarily.
BEGIN { $W = $^W; $^W = 0; }
use Sys::Syslog;
BEGIN { $^W = $W; }

use ASM::Messages;
use ASM::MessageCodes;
use strict;

# facility local0 level = info -> asm_audit.log
# facility local0 level = err -> asm_error.log
# facility local1 level = debug -> asm_trace.log
# facility local1 level = raw -> asm_raw.log  (All info and err messages)

my $msgcat;
my $facility = '';
my $object;
my $program = $0;
$program =~ s|.*/||;		# strip path from program
my $admin;

# This gets executed the first time
# any of the documented methods are called.
sub _init {
  return if $msgcat;		# safe to call again

  Sys::Syslog::setlogsock('unix');
  chomp($admin = `/usr/bin/whoami`);
  
  openlog("$facility:$program", # ident (set later using setFacility)
	  'pid',		# options
	  'local0');		# facility
  
  $msgcat = new ASM::Messages();
  unless (defined $msgcat) {
    # Don't make this fatal, it breaks perl syntax checking
    # because this code gets executed even when just running 'perl -c'
    # and our build machines don't have the message catalog installed.
    warn "$0: unable to open message catalog.\n";
  }
}

=head1 NAME

ASM::Log - Front end for logging ASM messages to the various log files

=head1 SYNOPSIS

  use ASM::Log;
  use ASM::MessageCodes;

  setAdmin($admin_name);
  setFacility('netconf');
  trace("changing hostname to $hostname... calling hostname...");
  error(MSG_ERR_HOSTNAME_FAILED, $hostname);
  audit(MSG_AUDIT_SET_HOSTNAME, $hostname);
  printf("ERROR: %s\n", getFormattedMessage(MSG_ERR_HOSTNAME_FAILED, $hostname));


=head1 DESCRIPTION

The B<ASM::Log> module allows ASM Perl scripts to log audit, error, and
trace messages without worrying about the details of how they work or where
they go.

The logs currently supported are:

=over 4

=item trace log

Primarily for our internal use, don\'t need to translate or document the
messages.

=item audit log

For user to be able to tell what has been done to the appliance via ASM,
and who did it.

Must be translated, and messages should include message codes.

=item error log

For user to get more information when a failed command is reported
through the Web interface.  Should be translated.  Should include more
complete information on what failed, e.g. filenames, etc.  But this
should still be a level above the raw output of things (put that in
the trace).

Try to include recovery instructions if feasible.

=back

=head1 DETAILS

The logs are all implemented by writing to the syslog facility, 
with different priorities:

=over 4

=item audit: priority=info   (translated and raw)

=item error: priority=err    (translated and raw)

=item trace: priority=debug  (untranslated only)

=back

Further, translated messages are written using the syslog
B<local0> facility, while raw messages are written using the
B<local1> facility.

This gives us the option, without changing code, of logging to one file
or many, just by changing F<syslog.conf>.  E.g.

 local0.=info		/var/log/asm_audit.log     # translated audit
 local0.=err		/var/log/asm_error.log     # translated errors
 local1.=debug		/var/log/asm_trace.log     # trace only

or

 local0.*,local1.*		/var/log/asmEverything.log

This does B<not> support multi-language logs, though; the audit and error logs
will contain messages in whatever language is determined by the current locale
as the scripts run.

=head1 METHODS

=over 4

=item setAdmin('adminname')

Sets the admin name to use in log messages.  If not set,
defaults to the Unix user id, probably 'root' or 'nobody'.

=cut

sub setAdmin {
  $admin = shift;
}

=item setFacility('facility name')

Set the ASM facility.  Legal values are:

  sysadmin sysuser netcfg backcfg backdata upgrade 
  syscfg security initsys initappl

See the ASM SDD, under "Creating Audit Information",
for more details of which facilities to use for what.

(Has nothing to do with the B<facility> in syslog, this is the
ASM facility.)

This is used as the syslog 'ident' value.

=cut

sub setFacility ($) {
  $facility = shift;
  $facility = "$facility:$program";

  # HACK ALERT - this seems to be the only way to change the syslog ident
  # other than re-opening syslog every time.  This is dependent
  # on the Sys::Syslog implementation and could break if Sys::Syslog
  # changes.
  $Sys::Syslog::ident    = "$facility";
}

=item setObject('object name')

Set the object being changed (optional).  For example, if updating
a user\'s password, use the user id; if updating a network interface,
use the interface name, etc.

=cut

sub setObject ($) {
  $object = shift;
}

=item trace('formatted message')

Log a complete message to the trace file. 

The admin and facility names are added, you don\'t need to include them.

  trace("The $which frangistan is pluking the quetzalator!");

=cut

sub trace ($) {
  my ($msg) = @_;
  asmlog('local1', 'debug', "$0: $msg");
  1;
}

=item audit(MSG_CODE [, 'arg1', 'arg2', ...])

Log a message to the audit log.  


Example:

  audit(MSG_HOSTNAME_CHANGED, $hostname);

might end up logging

=over 4

  AZM0004I netconf hostname administrator Changed hostname to web.ibm.com

=back

=cut

sub audit {
  asm_format_and_log('info', @_);
  asm_raw_message('info', @_);
  1;
}

=item error(MSG_CODE [, 'arg1', 'arg2', ...])

Log a message to the error log.  


Example:

  error(MSG_DISK_FULL, $mountname, $filename);

might end up logging

=over 4

  AZM5872E netconf disk administrator Disk /dev/sda10 mounted on /home is full.

=back

=cut

sub error {
  asm_format_and_log('err', @_);
  asm_raw_message('err', @_);
  1;
}

# Put backslashes in front of any quotes in the string
# Change real newslines to \n, carriage returns to \r,
# real backslashes to double backslash, etc.
sub escape_string ($) {
  my ($in) = @_;
  if ($in) {
    $in =~ s/\\/\\\\/g;
    $in =~ s/\"/\\"/g;        #" This comment just fixes emacs syntax highlighting
    $in =~ s/\n/\\n/g;
    $in =~ s/\r/\\r/g;
  }
  return $in;
}

sub asm_raw_message {
  my ($level, $msgcode, @args) = @_;
  # Format the "raw" message
  # Find out the name of the message code
  my $msgcode_name = $msgcode;
  foreach (@args) {
    if (!defined($_)) {
      $_ = "";
    }
  }
  my $msg = "\"" . join("\",\"", map { escape_string($_) } $msgcode_name, @args) . "\"";
  asmlog('local1', $level, $msg); # use local1 for all raw messages
}

sub asm_format_and_log {
  my ($level, $msgcode, @args) = @_;
  my $msg = getFormattedMessage($msgcode, @args);
  if ($level eq 'err') {
    # Also log to stderr
    print STDERR $msg, "\n";
  }
  asmlog('local0', $level, $msg); # use local1 for all translated messages
  1;
}

# Logs a pre-formatted message using the
# previously set ASM facility name as the Syslog "ident" value,
# the provided $syslog_facility as the Syslog 'facility' value 
#  (e.g. 'local0', 'local1', ...)
sub asmlog ($$$) {
  my ($syslog_facility, $level, $message) = @_;

   _init();

  if ($ASM::Messages::DEBUG) {	# if using testNLS
    print STDERR "asmlog: syslog_facility=$syslog_facility level=$level message=$message\n";
  }
  
  # HACK ALERT - this seems to be the only way to change the syslog facility
  # other than re-opening syslog every time.  This is dependent
  # on the Sys::Syslog implementation and could break if Sys::Syslog
  # changes.
  $Sys::Syslog::facility = $syslog_facility; # where to send it

  syslog($level, "[%s] %s", $admin, $message);
  1;
}

=item getFormattedMessage(MSG_CODE [, 'arg1', 'arg2', ...])

Retrieve, format a translated message, and return it

Example:

my $msg = getFormattedMessage(MSG_DISK_FULL, $mountname, $filename);

=cut

sub getFormattedMessage {
   my ($msgcode, @args) = @_;
   _init();
   return $msgcat->get($msgcode, @args);
}

=item getMimeCharset

Retrieve the MIME name of the charset that the messages will
be returned in.

Example:

$charset = getMimeCharset();

=cut

sub getMimeCharset {
   my $self = shift;
   return $msgcat->getMimeCharset();
}

=item changeLocale

changeLocale("localeName") - change the locale of the messages to
be "localeName".  e.g., ASM::Log::changeLocale("en_US") would change
to an English US message catalog.

=cut

sub changeLocale {
  my ($localeName) = shift;
  _init(); # Make sure we have a catalog already
  $msgcat->forceLocale($localeName);
  return;
}

=head1 SEE ALSO

ASM::MessageCodes where the message codes are defined.

Sys::Syslog which is used by this module.

=cut

1;
