#-*-perl-*-
#
# Licensed Materials - Property of IBM
# 
# 5724-A36
# 
# (C) Copyright IBM Corporation 2000. All Rights Reserved.
# 
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# Perl module to handle ASM message catalogs
#
# Hides whatever underlying mechanism we use
#
#

package ASM::Messages;
require Exporter;
@ISA       = qw(Exporter);
@EXPORT    = qw(new get close forceLocale );
@EXPORT_OK = qw(getMimeCharset);

require 5.004;
use locale;
use POSIX qw(locale_h);
use Carp;
use strict;
use ASM::BinHash;

use vars qw! $DEBUG !;

# From errno.h
use constant EBADF => 9;
use constant ENOMSG => 42;

#  From Common.pm
my $setup_data_file = "/opt/CSM/data/csm.data";

# This is used to force another locale if needed
my $forcedLocaleName = undef;

$DEBUG = 0;

=head1 NAME

ASM::Messages - Front end for managing translated messages in ASM

=head1 SYNOPSIS

  use ASM::Messages;
  
  $cat = new ASM::Messages();

  $msg = $cat->get(message_key [, args]);

  $cat->close();

=head1 DESCRIPTION

The B<ASM::Messages> module allows access to the message catalog functions
which are available on some systems. A new ASM::Messages object must first
be created for each catalog which has to be open at a given time.

The B<new> operation opens the catalog whose name is given as argument,
or the default ASM catalog.  (Most ASM scripts should use the default.)

The B<get> message retrieves the message with the given key (a string),
substituting parameters.

The B<close> function can be used when access to a catalog is not
needed anymore.  It is called automatically when the last reference to
the catalog goes out of scope.

=head1 EXAMPLES

  use ASM::Messages;
  use ASM::MessageCodes;

  $cat = new ASM::Messages() or do {
      print STDERR "Can't open message catalog.\n";
      exit(1);
  };
  printf "First message : %s\n", $cat->get(MSG_TEST_1);
  unless ($cat->close()) {
      print STDERR "Can't close whois catalog.\n";
      exit(1);
  }

=head1 METHODS

=over 4

=item $cat = new ASM::Messages();

=item $cat = new ASM::Messages("catname");

=item $cat = new ASM::Messages("/path/to/catalog");

Open a new catalog, returning the new catalog object.  With no argument,
opens the default ASM message catalog.  With a bare name, opens the named
catalog.  In either case, some unspecified magic is done to try to open
a catalog for the appropriate language.  YOU MUST HAVE CHOSEN A LOCALE
BEFORE CALLING OPEN!

With a full path, just tries to open the named file as a message catalog.

If an error occurs, returns B<undef>.

=cut

sub new {
  my $class = shift;
  my $self = {};
  bless $self, $class;
  return $self->_initialize(@_);
}

#  Following probably belong in Common.pm.
#  Must reorganize Common.pm, Log.pm, and Messages.pm to do this.

# Get the ASM Locale.
sub getASMLocale ()
{
   return($forcedLocaleName || getASMData("Language"));
}

sub getASMData($)
{
   my ($findkey) = @_;
   my ($foundvalue) = "";
   my ($line, $key, $value);

   open(DATAFILE,"< $setup_data_file") or warn("Couldn't open $setup_data_file");

   while ($line = <DATAFILE>)
   {
     next if $line =~ /^#/;
       ($key,$value) = split /=/,$line;
       chomp($value);
       if ($key eq $findkey)
	 {
	   $foundvalue = $value;
	   last;
	 }
   }

    close(DATAFILE);

    return ($foundvalue);
}


sub _initialize {
  my $self = shift;
  my $asmlocale = &getASMLocale();
  my $hashfile;
  if (-f "/opt/CSM/data/ASM-$asmlocale.binhash") {
    $hashfile = "/opt/CSM/data/ASM-$asmlocale.binhash";
  } elsif (-f  "/opt/CSM/data/ASM-en_US.binhash") {
    $hashfile = "/opt/CSM/data/ASM-en_US.binhash";
  } else {
    warn "Cannot find message file /opt/CSM/data/ASM-$asmlocale.binhash";
    my %dummy;
    $self->{hash} = \%dummy;
    return $self;
  }
  my ($href, $encoding) = readHash($hashfile);
  $self->{hash} = $href;
  $self->{encoding} = $encoding;
  return $self;
}

sub getMimeCharset {
  my $self = shift;
  if ($self->{encoding}) {
    my $encoding = $self->{encoding}; #  redlly the Java encoding name
    if ($encoding =~ /iso-?8859[-_]1$/i) {
      return "ISO-8859-1";
    } elsif ($encoding =~ /shift_jis/i) {
      return "Shift_JIS";
    } elsif ($encoding =~ /gb2312/i) {
      return "GB2312";
    } elsif ($encoding =~ /big5/i) {
      return "Big5";
    }
    return $encoding;
  } else {
    return "us-ascii";
  }
}

sub DESTROY {
  my $self = shift;
}

=item get(key [, parm1, parm2, ...])

The B<get> method retrieves the message with the given key (a string).

ASM code should always 'use ASM::MessageCodes' and refer to keys using
the constants defined in that module; ASM code should never pass a
string to B<get> directly.

If the retrieved string contains instances of {0}, {1}, {2}, etc they
are replaced with the given arguments, if present.  If no argument is
specified for a given replacement string, it is left unchanged in
the returned string.

If no message is found for the given key, then a string is returned
made up from the message key and the parameters; its exact form is not
specified, but it might look something like
"***MSG_KEY(PARM1,PARM2)***".

=cut

sub get {
  my $self = shift;
  my $key = shift;
  my $string;


  $string = $self->{hash}->{$key};
  if ( $string) {
    return replace($string, @_);
  } else {
    return "***" . $key . "(" . join(",", @_) . ")***";
  }

}

=item close

close - close the message catalog.  Don't use it anymore after calling this.

=cut

sub close {
  my $self = shift;
}

=item forceLocale

forceLocale("localeName") - change the locale of the messages to
be "localeName".  e.g., $catalog->forceLocale("en_US") would change
to an English US message catalog.

=cut

sub forceLocale {
  my ($self) = shift;
  my ($localeName) = shift;
  $forcedLocaleName = $localeName;
  $self->_initialize();
  return;
}

#
# Internal only
#
# Replace {0}, {1}, etc in the input string with the given parameters
#
sub replace {
  my ($in, @args) = @_;
  my $i = 0;
  while (@args) {
    my $arg = shift @args;
    if ($arg) {
      $in =~ s/\{$i\}/$arg/gm;
    } else {
      $in =~ s/\{$i\}//gm;
    }
    ++$i;
  }
  return $in;
}

=back

=head1 AUTHOR

Dan Poirier, poirier@us.ibm.com

=head1 SEE ALSO

catopen(3), catclose(3), catgets(3), perl(1).

Run "perldoc perllocale" to find out more about locales in Perl.

ASM::MessageCodes where the message codes are defined.

=cut

1;
